@@ -9,7 +9,14 @@ os.environ.setdefault("DJANGO_SETTINGS_MODULE", "config.settings")
 
 # When running on production-oriented nodes, avoid Celery debug mode.
 NODE_ROLE = os.environ.get("NODE_ROLE", "")
-if NODE_ROLE in {"Constellation", "Satellite"}:
+PRODUCTION_ROLES = {
+    "constellation",
+    "satellite",
+    "control",
+    "terminal",
+    "gateway",
+}
+if NODE_ROLE.lower() in PRODUCTION_ROLES:
     for var in ["CELERY_TRACE_APP", "CELERY_DEBUG"]:
         os.environ.pop(var, None)
     os.environ.setdefault("CELERY_LOG_LEVEL", "INFO")
