@@ -134,14 +134,33 @@ urlpatterns = [
         core_views.todo_done,
         name="todo-done",
     ),
+    path(
+        "admin/core/todos/<int:pk>/delete/",
+        core_views.todo_delete,
+        name="todo-delete",
+    ),
+    path(
+        "admin/core/todos/<int:pk>/snapshot/",
+        core_views.todo_snapshot,
+        name="todo-snapshot",
+    ),
     path(
         "admin/core/odoo-products/",
         core_views.odoo_products,
         name="odoo-products",
     ),
+    path(
+        "admin/core/odoo-quote-report/",
+        core_views.odoo_quote_report,
+        name="odoo-quote-report",
+    ),
+    path(
+        "admin/request-temp-password/",
+        core_views.request_temp_password,
+        name="admin-request-temp-password",
+    ),
     path("admin/", admin.site.urls),
     path("i18n/setlang/", csrf_exempt(set_language), name="set_language"),
-    path("api/", include("core.workgroup_urls")),
     path("", include("pages.urls")),
 ]
 
