@@ -4,10 +4,14 @@ import logging
 from django.contrib.auth.models import UserManager as DjangoUserManager
 from django.core.exceptions import FieldDoesNotExist
 from django.db import models
+from django.dispatch import Signal
 
 logger = logging.getLogger(__name__)
 
 
+user_data_flag_updated = Signal()
+
+
 class EntityQuerySet(models.QuerySet):
     def delete(self):  # pragma: no cover - delegates to instance delete
         deleted = 0
@@ -16,12 +20,24 @@ class EntityQuerySet(models.QuerySet):
             deleted += 1
         return deleted, {}
 
+    def update(self, **kwargs):
+        invalidate_user_data_cache = "is_user_data" in kwargs
+        updated = super().update(**kwargs)
+        if invalidate_user_data_cache and updated:
+            user_data_flag_updated.send(sender=self.model)
+        return updated
+
 
 class EntityManager(models.Manager):
     def get_queryset(self):
         return EntityQuerySet(self.model, using=self._db).filter(is_deleted=False)
 
 
+class EntityAllManager(models.Manager):
+    def get_queryset(self):
+        return EntityQuerySet(self.model, using=self._db)
+
+
 class EntityUserManager(DjangoUserManager):
     def get_queryset(self):
         return EntityQuerySet(self.model, using=self._db).filter(is_deleted=False)
@@ -35,7 +51,7 @@ class Entity(models.Model):
     is_deleted = models.BooleanField(default=False, editable=False)
 
     objects = EntityManager()
-    all_objects = models.Manager()
+    all_objects = EntityAllManager()
 
     class Meta:
         abstract = True
