@@ -1,20 +1,17 @@
 from __future__ import annotations
 
 import logging
+import shutil
+import re
 import subprocess
 from pathlib import Path
 import urllib.error
 import urllib.request
 
 from celery import shared_task
-from django.conf import settings
-from django.contrib.auth import get_user_model
-from core import mailer
 from core import github_issues
 from django.utils import timezone
 
-from nodes.models import NetMessage
-
 
 AUTO_UPGRADE_HEALTH_DELAY_SECONDS = 30
 AUTO_UPGRADE_SKIP_LOCK_NAME = "auto_upgrade_skip_revisions.lck"
@@ -29,23 +26,6 @@ def heartbeat() -> None:
     logger.info("Heartbeat task executed")
 
 
-@shared_task
-def birthday_greetings() -> None:
-    """Send birthday greetings to users via Net Message and email."""
-    User = get_user_model()
-    today = timezone.localdate()
-    for user in User.objects.filter(birthday=today):
-        NetMessage.broadcast("Happy bday!", user.username)
-        if user.email:
-            mailer.send(
-                "Happy bday!",
-                f"Happy bday! {user.username}",
-                [user.email],
-                settings.DEFAULT_FROM_EMAIL,
-                fail_silently=True,
-            )
-
-
 def _auto_upgrade_log_path(base_dir: Path) -> Path:
     """Return the log file used for auto-upgrade events."""
 
@@ -114,13 +94,30 @@ def _resolve_service_url(base_dir: Path) -> str:
     mode = "internal"
     if mode_file.exists():
         try:
-            mode = mode_file.read_text().strip() or "internal"
+            value = mode_file.read_text(encoding="utf-8").strip()
         except OSError:
-            mode = "internal"
+            value = ""
+        if value:
+            mode = value.lower()
     port = 8000 if mode == "public" else 8888
     return f"http://127.0.0.1:{port}/"
 
 
+def _parse_major_minor(version: str) -> tuple[int, int] | None:
+    match = re.match(r"^\s*(\d+)\.(\d+)", version)
+    if not match:
+        return None
+    return int(match.group(1)), int(match.group(2))
+
+
+def _shares_stable_series(local: str, remote: str) -> bool:
+    local_parts = _parse_major_minor(local)
+    remote_parts = _parse_major_minor(remote)
+    if not local_parts or not remote_parts:
+        return False
+    return local_parts == remote_parts
+
+
 @shared_task
 def check_github_updates() -> None:
     """Check the GitHub repo for updates and upgrade if needed."""
@@ -128,7 +125,16 @@ def check_github_updates() -> None:
     mode_file = base_dir / "locks" / "auto_upgrade.lck"
     mode = "version"
     if mode_file.exists():
-        mode = mode_file.read_text().strip()
+        try:
+            raw_mode = mode_file.read_text().strip()
+        except (OSError, UnicodeDecodeError):
+            logger.warning(
+                "Failed to read auto-upgrade mode lockfile", exc_info=True
+            )
+        else:
+            cleaned_mode = raw_mode.lower()
+            if cleaned_mode:
+                mode = cleaned_mode
 
     branch = "main"
     subprocess.run(["git", "fetch", "origin", branch], cwd=base_dir, check=True)
@@ -206,9 +212,16 @@ def check_github_updates() -> None:
             if startup:
                 startup()
             return
+        if mode == "stable" and _shares_stable_series(local, remote):
+            if startup:
+                startup()
+            return
         if notify:
             notify("Upgrading...", upgrade_stamp)
-        args = ["./upgrade.sh", "--no-restart"]
+        if mode == "stable":
+            args = ["./upgrade.sh", "--stable", "--no-restart"]
+        else:
+            args = ["./upgrade.sh", "--no-restart"]
         upgrade_was_applied = True
 
     with log_file.open("a") as fh:
