@@ -6,6 +6,6 @@ urlpatterns = [
     path("rfid-login/", views.rfid_login, name="rfid-login"),
     path("rfids/", views.rfid_batch, name="rfid-batch"),
     path("products/", views.product_list, name="product-list"),
-    path("subscribe/", views.add_subscription, name="add-subscription"),
-    path("list/", views.subscription_list, name="subscription-list"),
+    path("live-subscribe/", views.add_live_subscription, name="add-live-subscription"),
+    path("live-list/", views.live_subscription_list, name="live-subscription-list"),
 ]
