@@ -1,6 +1,7 @@
 import logging
 import os
 import socket
+import sys
 import threading
 import time
 from pathlib import Path
@@ -31,6 +32,21 @@ def _startup_notification() -> None:
     rev_short = revision_value[-6:] if revision_value else ""
 
     body = version
+    if body:
+        normalized = body.lstrip("vV") or body
+        base_version = normalized.rstrip("+")
+        needs_marker = False
+        if base_version and revision_value:
+            try:  # pragma: no cover - defensive guard
+                from core.models import PackageRelease
+
+                needs_marker = not PackageRelease.matches_revision(
+                    base_version, revision_value
+                )
+            except Exception:
+                logger.debug("Startup release comparison failed", exc_info=True)
+        if needs_marker and not normalized.endswith("+"):
+            body = f"{body}+"
     if rev_short:
         body = f"{body} r{rev_short}" if body else f"r{rev_short}"
 
@@ -51,6 +67,10 @@ def _startup_notification() -> None:
 def _trigger_startup_notification(**_: object) -> None:
     """Attempt to send the startup notification in the background."""
 
+    if _is_running_migration_command():
+        logger.debug("Startup notification skipped: running migration command")
+        return
+
     try:
         connections["default"].ensure_connection()
     except OperationalError:
@@ -59,6 +79,12 @@ def _trigger_startup_notification(**_: object) -> None:
     _startup_notification()
 
 
+def _is_running_migration_command() -> bool:
+    """Return ``True`` when Django's ``migrate`` command is executing."""
+
+    return len(sys.argv) > 1 and sys.argv[1] == "migrate"
+
+
 class NodesConfig(AppConfig):
     default_auto_field = "django.db.models.BigAutoField"
     name = "nodes"
@@ -68,3 +94,5 @@ class NodesConfig(AppConfig):
         from django.db.models.signals import post_migrate
 
         post_migrate.connect(_trigger_startup_notification, sender=self)
+        # Import signal handlers for content classifiers
+        from . import signals  # noqa: F401
