@@ -4,12 +4,14 @@ import hashlib
 import logging
 import shutil
 import subprocess
+import uuid
 
 from django.conf import settings
 from selenium import webdriver
 from selenium.webdriver.firefox.options import Options
 from selenium.common.exceptions import WebDriverException
 
+from .classifiers import run_default_classifiers, suppress_default_classifiers
 from .models import ContentSample
 
 SCREENSHOT_DIR = settings.LOG_DIR / "screenshots"
@@ -56,7 +58,9 @@ def capture_rpi_snapshot(timeout: int = 10) -> Path:
     if not tool_path:
         raise RuntimeError("rpicam-still is not available")
     CAMERA_DIR.mkdir(parents=True, exist_ok=True)
-    filename = CAMERA_DIR / f"{datetime.utcnow():%Y%m%d%H%M%S}.jpg"
+    timestamp = datetime.utcnow()
+    unique_suffix = uuid.uuid4().hex
+    filename = CAMERA_DIR / f"{timestamp:%Y%m%d%H%M%S}-{unique_suffix}.jpg"
     try:
         result = subprocess.run(
             [tool_path, "-o", str(filename), "-t", "1"],
@@ -78,7 +82,15 @@ def capture_rpi_snapshot(timeout: int = 10) -> Path:
     return filename
 
 
-def save_screenshot(path: Path, node=None, method: str = "", transaction_uuid=None):
+def save_screenshot(
+    path: Path,
+    node=None,
+    method: str = "",
+    transaction_uuid=None,
+    *,
+    content: str | None = None,
+    user=None,
+):
     """Save screenshot file info if not already recorded.
 
     Returns the created :class:`ContentSample` or ``None`` if duplicate.
@@ -102,4 +114,11 @@ def save_screenshot(path: Path, node=None, method: str = "", transaction_uuid=No
     }
     if transaction_uuid is not None:
         data["transaction_uuid"] = transaction_uuid
-    return ContentSample.objects.create(**data)
+    if content is not None:
+        data["content"] = content
+    if user is not None:
+        data["user"] = user
+    with suppress_default_classifiers():
+        sample = ContentSample.objects.create(**data)
+    run_default_classifiers(sample)
+    return sample
