@@ -2,11 +2,12 @@ from django.contrib import admin, messages
 from django import forms
 
 import asyncio
-from datetime import timedelta
+from datetime import datetime, time, timedelta
 import json
 
 from django.shortcuts import redirect
-from django.utils import timezone
+from django.utils import formats, timezone, translation
+from django.utils.html import format_html
 from django.urls import path
 from django.http import HttpResponse, HttpResponseRedirect
 from django.template.response import TemplateResponse
@@ -16,6 +17,7 @@ from asgiref.sync import async_to_sync
 
 from .models import (
     Charger,
+    ChargerConfiguration,
     Simulator,
     MeterValue,
     Transaction,
@@ -28,6 +30,9 @@ from .transactions_io import (
     export_transactions,
     import_transactions as import_transactions_data,
 )
+from .status_display import STATUS_BADGE_MAP, ERROR_OK_VALUES
+from .views import _charger_state, _live_sessions
+from core.admin import SaveBeforeChangeAction
 from core.user_data import EntityModelAdmin
 
 
@@ -105,6 +110,101 @@ class LogViewAdminMixin:
         return TemplateResponse(request, self.log_template_name, context)
 
 
+@admin.register(ChargerConfiguration)
+class ChargerConfigurationAdmin(admin.ModelAdmin):
+    list_display = (
+        "charger_identifier",
+        "connector_display",
+        "origin_display",
+        "created_at",
+    )
+    list_filter = ("connector_id",)
+    search_fields = ("charger_identifier",)
+    readonly_fields = (
+        "charger_identifier",
+        "connector_id",
+        "origin_display",
+        "evcs_snapshot_at",
+        "created_at",
+        "updated_at",
+        "linked_chargers",
+        "configuration_keys_display",
+        "unknown_keys_display",
+        "raw_payload_display",
+    )
+    fieldsets = (
+        (
+            None,
+            {
+                "fields": (
+                    "charger_identifier",
+                    "connector_id",
+                    "origin_display",
+                    "evcs_snapshot_at",
+                    "linked_chargers",
+                    "created_at",
+                    "updated_at",
+                )
+            },
+        ),
+        (
+            "Payload",
+            {
+                "fields": (
+                    "configuration_keys_display",
+                    "unknown_keys_display",
+                    "raw_payload_display",
+                )
+            },
+        ),
+    )
+
+    @admin.display(description="Connector")
+    def connector_display(self, obj):
+        if obj.connector_id is None:
+            return "All"
+        return obj.connector_id
+
+    @admin.display(description="Linked charge points")
+    def linked_chargers(self, obj):
+        if obj.pk is None:
+            return ""
+        linked = [charger.identity_slug() for charger in obj.chargers.all()]
+        if not linked:
+            return "-"
+        return ", ".join(sorted(linked))
+
+    def _render_json(self, data):
+        from django.utils.html import format_html
+
+        if not data:
+            return "-"
+        formatted = json.dumps(data, indent=2, ensure_ascii=False)
+        return format_html("<pre>{}</pre>", formatted)
+
+    @admin.display(description="configurationKey")
+    def configuration_keys_display(self, obj):
+        return self._render_json(obj.configuration_keys)
+
+    @admin.display(description="unknownKey")
+    def unknown_keys_display(self, obj):
+        return self._render_json(obj.unknown_keys)
+
+    @admin.display(description="Raw payload")
+    def raw_payload_display(self, obj):
+        return self._render_json(obj.raw_payload)
+
+    @admin.display(description="Origin")
+    def origin_display(self, obj):
+        if obj.evcs_snapshot_at:
+            return "EVCS"
+        return "Local"
+
+    def save_model(self, request, obj, form, change):
+        obj.evcs_snapshot_at = None
+        super().save_model(request, obj, form, change)
+
+
 @admin.register(Location)
 class LocationAdmin(EntityModelAdmin):
     form = LocationAdminForm
@@ -160,6 +260,7 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
                     "charger_id",
                     "display_name",
                     "connector_id",
+                    "language",
                     "location",
                     "last_path",
                     "last_heartbeat",
@@ -203,7 +304,7 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
         ),
         (
             "Configuration",
-            {"fields": ("public_display", "require_rfid")},
+            {"fields": ("public_display", "require_rfid", "configuration")},
         ),
         (
             "References",
@@ -232,11 +333,12 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
         "availability_request_status",
         "availability_request_status_at",
         "availability_request_details",
+        "configuration",
     )
     list_display = (
-        "charger_id",
+        "display_name_with_fallback",
         "connector_number",
-        "location_name",
+        "charger_name_display",
         "require_rfid_display",
         "public_display",
         "last_heartbeat",
@@ -251,10 +353,14 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
     actions = [
         "purge_data",
         "fetch_cp_configuration",
+        "toggle_rfid_authentication",
+        "recheck_charger_status",
         "change_availability_operative",
         "change_availability_inoperative",
         "set_availability_state_operative",
         "set_availability_state_inoperative",
+        "remote_stop_transaction",
+        "reset_chargers",
         "delete_selected",
     ]
 
@@ -313,11 +419,49 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
             "charger-status-connector",
             args=[obj.charger_id, obj.connector_slug],
         )
-        label = (obj.last_status or "status").strip() or "status"
-        return format_html('<a href="{}" target="_blank">{}</a>', url, label)
+        tx_obj = store.get_transaction(obj.charger_id, obj.connector_id)
+        state, _ = _charger_state(
+            obj,
+            tx_obj
+            if obj.connector_id is not None
+            else (_live_sessions(obj) or None),
+        )
+        return format_html('<a href="{}" target="_blank">{}</a>', url, state)
 
     status_link.short_description = "Status"
 
+    def _has_active_session(self, charger: Charger) -> bool:
+        """Return whether ``charger`` currently has an active session."""
+
+        if store.get_transaction(charger.charger_id, charger.connector_id):
+            return True
+        if charger.connector_id is not None:
+            return False
+        sibling_connectors = (
+            Charger.objects.filter(charger_id=charger.charger_id)
+            .exclude(pk=charger.pk)
+            .values_list("connector_id", flat=True)
+        )
+        for connector_id in sibling_connectors:
+            if store.get_transaction(charger.charger_id, connector_id):
+                return True
+        return False
+
+    @admin.display(description="Display Name", ordering="display_name")
+    def display_name_with_fallback(self, obj):
+        return self._charger_display_name(obj)
+
+    @admin.display(description="Charger", ordering="display_name")
+    def charger_name_display(self, obj):
+        return self._charger_display_name(obj)
+
+    def _charger_display_name(self, obj):
+        if obj.display_name:
+            return obj.display_name
+        if obj.location:
+            return obj.location.name
+        return obj.charger_id
+
     def location_name(self, obj):
         return obj.location.name if obj.location else ""
 
@@ -330,6 +474,63 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
 
     purge_data.short_description = "Purge data"
 
+    @admin.action(description="Re-check Charger Status")
+    def recheck_charger_status(self, request, queryset):
+        requested = 0
+        for charger in queryset:
+            connector_value = charger.connector_id
+            ws = store.get_connection(charger.charger_id, connector_value)
+            if ws is None:
+                self.message_user(
+                    request,
+                    f"{charger}: no active connection",
+                    level=messages.ERROR,
+                )
+                continue
+            payload: dict[str, object] = {"requestedMessage": "StatusNotification"}
+            trigger_connector: int | None = None
+            if connector_value is not None:
+                payload["connectorId"] = connector_value
+                trigger_connector = connector_value
+            message_id = uuid.uuid4().hex
+            msg = json.dumps([2, message_id, "TriggerMessage", payload])
+            try:
+                async_to_sync(ws.send)(msg)
+            except Exception as exc:  # pragma: no cover - network error
+                self.message_user(
+                    request,
+                    f"{charger}: failed to send TriggerMessage ({exc})",
+                    level=messages.ERROR,
+                )
+                continue
+            log_key = store.identity_key(charger.charger_id, connector_value)
+            store.add_log(log_key, f"< {msg}", log_type="charger")
+            store.register_pending_call(
+                message_id,
+                {
+                    "action": "TriggerMessage",
+                    "charger_id": charger.charger_id,
+                    "connector_id": connector_value,
+                    "log_key": log_key,
+                    "trigger_target": "StatusNotification",
+                    "trigger_connector": trigger_connector,
+                    "requested_at": timezone.now(),
+                },
+            )
+            store.schedule_call_timeout(
+                message_id,
+                timeout=5.0,
+                action="TriggerMessage",
+                log_key=log_key,
+                message="TriggerMessage StatusNotification timed out",
+            )
+            requested += 1
+        if requested:
+            self.message_user(
+                request,
+                f"Requested status update from {requested} charger(s)",
+            )
+
     @admin.action(description="Fetch CP configuration")
     def fetch_cp_configuration(self, request, queryset):
         fetched = 0
@@ -384,6 +585,30 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
                 f"Requested configuration from {fetched} charger(s)",
             )
 
+    @admin.action(description="Toggle RFID Authentication")
+    def toggle_rfid_authentication(self, request, queryset):
+        enabled = 0
+        disabled = 0
+        for charger in queryset:
+            new_value = not charger.require_rfid
+            Charger.objects.filter(pk=charger.pk).update(require_rfid=new_value)
+            charger.require_rfid = new_value
+            if new_value:
+                enabled += 1
+            else:
+                disabled += 1
+        if enabled or disabled:
+            changes = []
+            if enabled:
+                changes.append(f"enabled for {enabled} charger(s)")
+            if disabled:
+                changes.append(f"disabled for {disabled} charger(s)")
+            summary = "; ".join(changes)
+            self.message_user(
+                request,
+                f"Updated RFID authentication: {summary}",
+            )
+
     def _dispatch_change_availability(self, request, queryset, availability_type: str):
         sent = 0
         for charger in queryset:
@@ -475,6 +700,112 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
     def set_availability_state_inoperative(self, request, queryset):
         self._set_availability_state(request, queryset, "Inoperative")
 
+    @admin.action(description="Remote stop active transaction")
+    def remote_stop_transaction(self, request, queryset):
+        stopped = 0
+        for charger in queryset:
+            connector_value = charger.connector_id
+            ws = store.get_connection(charger.charger_id, connector_value)
+            if ws is None:
+                self.message_user(
+                    request,
+                    f"{charger}: no active connection",
+                    level=messages.ERROR,
+                )
+                continue
+            tx_obj = store.get_transaction(charger.charger_id, connector_value)
+            if tx_obj is None:
+                self.message_user(
+                    request,
+                    f"{charger}: no active transaction",
+                    level=messages.ERROR,
+                )
+                continue
+            message_id = uuid.uuid4().hex
+            payload = {"transactionId": tx_obj.pk}
+            msg = json.dumps([
+                2,
+                message_id,
+                "RemoteStopTransaction",
+                payload,
+            ])
+            try:
+                async_to_sync(ws.send)(msg)
+            except Exception as exc:  # pragma: no cover - network error
+                self.message_user(
+                    request,
+                    f"{charger}: failed to send RemoteStopTransaction ({exc})",
+                    level=messages.ERROR,
+                )
+                continue
+            log_key = store.identity_key(charger.charger_id, connector_value)
+            store.add_log(log_key, f"< {msg}", log_type="charger")
+            store.register_pending_call(
+                message_id,
+                {
+                    "action": "RemoteStopTransaction",
+                    "charger_id": charger.charger_id,
+                    "connector_id": connector_value,
+                    "transaction_id": tx_obj.pk,
+                    "log_key": log_key,
+                    "requested_at": timezone.now(),
+                },
+            )
+            stopped += 1
+        if stopped:
+            self.message_user(
+                request,
+                f"Sent RemoteStopTransaction to {stopped} charger(s)",
+            )
+
+    @admin.action(description="Reset charger (soft)")
+    def reset_chargers(self, request, queryset):
+        reset = 0
+        for charger in queryset:
+            connector_value = charger.connector_id
+            ws = store.get_connection(charger.charger_id, connector_value)
+            if ws is None:
+                self.message_user(
+                    request,
+                    f"{charger}: no active connection",
+                    level=messages.ERROR,
+                )
+                continue
+            message_id = uuid.uuid4().hex
+            msg = json.dumps([
+                2,
+                message_id,
+                "Reset",
+                {"type": "Soft"},
+            ])
+            try:
+                async_to_sync(ws.send)(msg)
+            except Exception as exc:  # pragma: no cover - network error
+                self.message_user(
+                    request,
+                    f"{charger}: failed to send Reset ({exc})",
+                    level=messages.ERROR,
+                )
+                continue
+            log_key = store.identity_key(charger.charger_id, connector_value)
+            store.add_log(log_key, f"< {msg}", log_type="charger")
+            store.register_pending_call(
+                message_id,
+                {
+                    "action": "Reset",
+                    "charger_id": charger.charger_id,
+                    "connector_id": connector_value,
+                    "log_key": log_key,
+                    "requested_at": timezone.now(),
+                },
+            )
+            reset += 1
+        if reset:
+            self.message_user(
+                request,
+                f"Sent Reset to {reset} charger(s)",
+            )
+
     def delete_queryset(self, request, queryset):
         for obj in queryset:
             obj.delete()
@@ -492,9 +823,47 @@ class ChargerAdmin(LogViewAdminMixin, EntityModelAdmin):
 
     session_kw.short_description = "Session kW"
 
+    def changelist_view(self, request, extra_context=None):
+        response = super().changelist_view(request, extra_context=extra_context)
+        if hasattr(response, "context_data"):
+            cl = response.context_data.get("cl")
+            if cl is not None:
+                response.context_data.update(
+                    self._charger_quick_stats_context(cl.queryset)
+                )
+        return response
+
+    def _charger_quick_stats_context(self, queryset):
+        chargers = list(queryset)
+        stats = {"total_kw": 0.0, "today_kw": 0.0}
+        if not chargers:
+            return {"charger_quick_stats": stats}
+
+        parent_ids = {c.charger_id for c in chargers if c.connector_id is None}
+        start, end = self._today_range()
+
+        for charger in chargers:
+            include_totals = True
+            if charger.connector_id is not None and charger.charger_id in parent_ids:
+                include_totals = False
+            if include_totals:
+                stats["total_kw"] += charger.total_kw
+                stats["today_kw"] += charger.total_kw_for_range(start, end)
+
+        stats = {key: round(value, 2) for key, value in stats.items()}
+        return {"charger_quick_stats": stats}
+
+    def _today_range(self):
+        today = timezone.localdate()
+        start = datetime.combine(today, time.min)
+        if timezone.is_naive(start):
+            start = timezone.make_aware(start, timezone.get_current_timezone())
+        end = start + timedelta(days=1)
+        return start, end
+
 
 @admin.register(Simulator)
-class SimulatorAdmin(LogViewAdminMixin, EntityModelAdmin):
+class SimulatorAdmin(SaveBeforeChangeAction, LogViewAdminMixin, EntityModelAdmin):
     list_display = (
         "name",
         "cp_path",
@@ -502,7 +871,7 @@ class SimulatorAdmin(LogViewAdminMixin, EntityModelAdmin):
         "ws_port",
         "ws_url",
         "interval",
-        "kw_max",
+        "kw_max_display",
         "running",
         "log_link",
     )
@@ -538,9 +907,30 @@ class SimulatorAdmin(LogViewAdminMixin, EntityModelAdmin):
         ),
     )
     actions = ("start_simulator", "stop_simulator", "send_open_door")
+    change_actions = ["start_simulator_action", "stop_simulator_action"]
 
     log_type = "simulator"
 
+    @admin.display(description="kW Max", ordering="kw_max")
+    def kw_max_display(self, obj):
+        """Display ``kw_max`` with a dot decimal separator for Spanish locales."""
+
+        language = translation.get_language() or ""
+        if language.startswith("es"):
+            return formats.number_format(
+                obj.kw_max,
+                decimal_pos=2,
+                use_l10n=False,
+                force_grouping=False,
+            )
+
+        return formats.number_format(
+            obj.kw_max,
+            decimal_pos=2,
+            use_l10n=True,
+            force_grouping=False,
+        )
+
     def save_model(self, request, obj, form, change):
         previous_door_open = False
         if change and obj.pk:
@@ -650,6 +1040,14 @@ class SimulatorAdmin(LogViewAdminMixin, EntityModelAdmin):
 
     stop_simulator.short_description = "Stop selected simulators"
 
+    def start_simulator_action(self, request, obj):
+        queryset = type(obj).objects.filter(pk=obj.pk)
+        self.start_simulator(request, queryset)
+
+    def stop_simulator_action(self, request, obj):
+        queryset = type(obj).objects.filter(pk=obj.pk)
+        self.stop_simulator(request, queryset)
+
     def log_link(self, obj):
         from django.utils.html import format_html
         from django.urls import reverse
@@ -686,19 +1084,27 @@ class TransactionAdmin(EntityModelAdmin):
     change_list_template = "admin/ocpp/transaction/change_list.html"
     list_display = (
         "charger",
+        "connector_number",
         "account",
         "rfid",
+        "vid",
         "meter_start",
         "meter_stop",
         "start_time",
         "stop_time",
         "kw",
     )
-    readonly_fields = ("kw",)
+    readonly_fields = ("kw", "received_start_time", "received_stop_time")
     list_filter = ("charger", "account")
     date_hierarchy = "start_time"
     inlines = [MeterValueInline]
 
+    def connector_number(self, obj):
+        return obj.connector_id or ""
+
+    connector_number.short_description = "#"
+    connector_number.admin_order_field = "connector_id"
+
     def get_urls(self):
         urls = super().get_urls()
         custom = [
