@@ -5,6 +5,8 @@ from datetime import datetime
 import asyncio
 import inspect
 import json
+import logging
+from urllib.parse import parse_qs
 from django.utils import timezone
 from core.models import EnergyAccount, Reference, RFID as CoreRFID
 from nodes.models import NetMessage
@@ -18,7 +20,13 @@ from config.offline import requires_network
 from . import store
 from decimal import Decimal
 from django.utils.dateparse import parse_datetime
-from .models import Transaction, Charger, MeterValue, DataTransferMessage
+from .models import (
+    Transaction,
+    Charger,
+    ChargerConfiguration,
+    MeterValue,
+    DataTransferMessage,
+)
 from .reference_utils import host_is_local_loopback
 from .evcs_discovery import (
     DEFAULT_CONSOLE_PORT,
@@ -31,6 +39,21 @@ from .evcs_discovery import (
 FORWARDED_PAIR_RE = re.compile(r"for=(?:\"?)(?P<value>[^;,\"\s]+)(?:\"?)", re.IGNORECASE)
 
 
+logger = logging.getLogger(__name__)
+
+
+# Query parameter keys that may contain the charge point serial. Keys are
+# matched case-insensitively and trimmed before use.
+SERIAL_QUERY_PARAM_NAMES = (
+    "cid",
+    "chargepointid",
+    "charge_point_id",
+    "chargeboxid",
+    "charge_box_id",
+    "chargerid",
+)
+
+
 def _parse_ip(value: str | None):
     """Return an :mod:`ipaddress` object for the provided value, if valid."""
 
@@ -105,6 +128,34 @@ def _resolve_client_ip(scope: dict) -> str | None:
     return fallback
 
 
+def _parse_ocpp_timestamp(value) -> datetime | None:
+    """Return an aware :class:`~datetime.datetime` for OCPP timestamps."""
+
+    if not value:
+        return None
+    if isinstance(value, datetime):
+        timestamp = value
+    else:
+        timestamp = parse_datetime(str(value))
+    if not timestamp:
+        return None
+    if timezone.is_naive(timestamp):
+        timestamp = timezone.make_aware(timestamp, timezone.get_current_timezone())
+    return timestamp
+
+
+def _extract_vehicle_identifier(payload: dict) -> tuple[str, str]:
+    """Return normalized VID and VIN values from an OCPP message payload."""
+
+    raw_vid = payload.get("vid")
+    vid_value = str(raw_vid).strip() if raw_vid is not None else ""
+    raw_vin = payload.get("vin")
+    vin_value = str(raw_vin).strip() if raw_vin is not None else ""
+    if not vid_value and vin_value:
+        vid_value = vin_value
+    return vid_value, vin_value
+
+
 class SinkConsumer(AsyncWebsocketConsumer):
     """Accept any message without validation."""
 
@@ -137,15 +188,53 @@ class CSMSConsumer(AsyncWebsocketConsumer):
 
     consumption_update_interval = 300
 
+    def _extract_serial_identifier(self) -> str:
+        """Return the charge point serial from the query string or path."""
+
+        self.serial_source = None
+        query_bytes = self.scope.get("query_string") or b""
+        self._raw_query_string = query_bytes.decode("utf-8", "ignore") if query_bytes else ""
+        if query_bytes:
+            try:
+                parsed = parse_qs(
+                    self._raw_query_string,
+                    keep_blank_values=False,
+                )
+            except Exception:
+                parsed = {}
+            if parsed:
+                normalized = {
+                    key.lower(): values for key, values in parsed.items() if values
+                }
+                for candidate in SERIAL_QUERY_PARAM_NAMES:
+                    values = normalized.get(candidate)
+                    if not values:
+                        continue
+                    for value in values:
+                        if not value:
+                            continue
+                        trimmed = value.strip()
+                        if trimmed:
+                            return trimmed
+                          
+        return self.scope["url_route"]["kwargs"].get("cid", "")
+
     @requires_network
     async def connect(self):
-        raw_serial = self.scope["url_route"]["kwargs"].get("cid", "")
+        raw_serial = self._extract_serial_identifier()
         try:
             self.charger_id = Charger.validate_serial(raw_serial)
         except ValidationError as exc:
             serial = Charger.normalize_serial(raw_serial)
             store_key = store.pending_key(serial)
             message = exc.messages[0] if exc.messages else "Invalid Serial Number"
+            details: list[str] = []
+            if getattr(self, "serial_source", None):
+                details.append(f"serial_source={self.serial_source}")
+            if getattr(self, "_raw_query_string", ""):
+                details.append(f"query_string={self._raw_query_string!r}")
+            if details:
+                message = f"{message} ({'; '.join(details)})"
             store.add_log(
                 store_key,
                 f"Rejected connection: {message}",
@@ -212,11 +301,55 @@ class CSMSConsumer(AsyncWebsocketConsumer):
         """Return the energy account for the provided RFID if valid."""
         if not id_tag:
             return None
-        return await database_sync_to_async(
-            EnergyAccount.objects.filter(
-                rfids__rfid=id_tag.upper(), rfids__allowed=True
-            ).first
-        )()
+
+        def _resolve() -> EnergyAccount | None:
+            matches = CoreRFID.matching_queryset(id_tag).filter(allowed=True)
+            if not matches.exists():
+                return None
+            return (
+                EnergyAccount.objects.filter(rfids__in=matches)
+                .distinct()
+                .first()
+            )
+
+        return await database_sync_to_async(_resolve)()
+
+    async def _ensure_rfid_seen(self, id_tag: str) -> CoreRFID | None:
+        """Ensure an RFID record exists and update its last seen timestamp."""
+
+        if not id_tag:
+            return None
+
+        normalized = id_tag.upper()
+
+        def _ensure() -> CoreRFID:
+            now = timezone.now()
+            tag, _created = CoreRFID.register_scan(normalized)
+            updates = []
+            if not tag.allowed:
+                tag.allowed = True
+                updates.append("allowed")
+            if tag.last_seen_on != now:
+                tag.last_seen_on = now
+                updates.append("last_seen_on")
+            if updates:
+                tag.save(update_fields=updates)
+            return tag
+
+        return await database_sync_to_async(_ensure)()
+
+    def _log_unlinked_rfid(self, rfid: str) -> None:
+        """Record a warning when an RFID is authorized without an account."""
+
+        message = (
+            f"Authorized RFID {rfid} on charger {self.charger_id} without linked energy account"
+        )
+        logger.warning(message)
+        store.add_log(
+            store.pending_key(self.charger_id),
+            message,
+            log_type="charger",
+        )
 
     async def _assign_connector(self, connector: int | str | None) -> None:
         """Ensure ``self.charger`` matches the provided connector id."""
@@ -230,6 +363,44 @@ class CSMSConsumer(AsyncWebsocketConsumer):
             except (TypeError, ValueError):
                 return
         if connector_value is None:
+            aggregate = self.aggregate_charger
+            if (
+                not aggregate
+                or aggregate.connector_id is not None
+                or aggregate.charger_id != self.charger_id
+            ):
+                aggregate, _ = await database_sync_to_async(
+                    Charger.objects.get_or_create
+                )(
+                    charger_id=self.charger_id,
+                    connector_id=None,
+                    defaults={"last_path": self.scope.get("path", "")},
+                )
+                await database_sync_to_async(aggregate.refresh_manager_node)()
+                self.aggregate_charger = aggregate
+            self.charger = self.aggregate_charger
+            previous_key = self.store_key
+            new_key = store.identity_key(self.charger_id, None)
+            if previous_key != new_key:
+                existing_consumer = store.connections.get(new_key)
+                if existing_consumer is not None and existing_consumer is not self:
+                    await existing_consumer.close()
+                store.reassign_identity(previous_key, new_key)
+                store.connections[new_key] = self
+                store.logs["charger"].setdefault(new_key, [])
+            aggregate_name = await sync_to_async(
+                lambda: self.charger.name or self.charger.charger_id
+            )()
+            friendly_name = aggregate_name or self.charger_id
+            store.register_log_name(new_key, friendly_name, log_type="charger")
+            store.register_log_name(
+                store.identity_key(self.charger_id, None),
+                friendly_name,
+                log_type="charger",
+            )
+            store.register_log_name(self.charger_id, friendly_name, log_type="charger")
+            self.store_key = new_key
+            self.connector_value = None
             if not self._header_reference_created and self.client_ip:
                 await database_sync_to_async(self._ensure_console_reference)()
                 self._header_reference_created = True
@@ -580,6 +751,54 @@ class CSMSConsumer(AsyncWebsocketConsumer):
         task.add_done_callback(lambda _: setattr(self, "_consumption_task", None))
         self._consumption_task = task
 
+    def _persist_configuration_result(
+        self, payload: dict, connector_hint: int | str | None
+    ) -> ChargerConfiguration | None:
+        if not isinstance(payload, dict):
+            return None
+
+        connector_value: int | None = None
+        if connector_hint not in (None, ""):
+            try:
+                connector_value = int(connector_hint)
+            except (TypeError, ValueError):
+                connector_value = None
+
+        normalized_entries: list[dict[str, object]] = []
+        for entry in payload.get("configurationKey") or []:
+            if not isinstance(entry, dict):
+                continue
+            key = str(entry.get("key") or "")
+            normalized: dict[str, object] = {"key": key}
+            if "value" in entry:
+                normalized["value"] = entry.get("value")
+            normalized["readonly"] = bool(entry.get("readonly"))
+            normalized_entries.append(normalized)
+
+        unknown_values: list[str] = []
+        for value in payload.get("unknownKey") or []:
+            if value is None:
+                continue
+            unknown_values.append(str(value))
+
+        try:
+            raw_payload = json.loads(json.dumps(payload, ensure_ascii=False))
+        except (TypeError, ValueError):
+            raw_payload = payload
+
+        configuration = ChargerConfiguration.objects.create(
+            charger_identifier=self.charger_id,
+            connector_id=connector_value,
+            configuration_keys=normalized_entries,
+            unknown_keys=unknown_values,
+            evcs_snapshot_at=timezone.now(),
+            raw_payload=raw_payload,
+        )
+        Charger.objects.filter(charger_id=self.charger_id).update(
+            configuration=configuration
+        )
+        return configuration
+
     async def _handle_call_result(self, message_id: str, payload: dict | None) -> None:
         metadata = store.pop_pending_call(message_id)
         if not metadata:
@@ -588,9 +807,15 @@ class CSMSConsumer(AsyncWebsocketConsumer):
             return
         action = metadata.get("action")
         log_key = metadata.get("log_key") or self.store_key
+        payload_data = payload if isinstance(payload, dict) else {}
         if action == "DataTransfer":
             message_pk = metadata.get("message_pk")
             if not message_pk:
+                store.record_pending_call_result(
+                    message_id,
+                    metadata=metadata,
+                    payload=payload_data,
+                )
                 return
 
             def _apply():
@@ -617,9 +842,13 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 )
 
             await database_sync_to_async(_apply)()
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         if action == "GetConfiguration":
-            payload_data = payload if isinstance(payload, dict) else {}
             try:
                 payload_text = json.dumps(
                     payload_data, sort_keys=True, ensure_ascii=False
@@ -631,9 +860,24 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 f"GetConfiguration result: {payload_text}",
                 log_type="charger",
             )
+            configuration = await database_sync_to_async(
+                self._persist_configuration_result
+            )(payload_data, metadata.get("connector_id"))
+            if configuration:
+                if self.charger and self.charger.charger_id == self.charger_id:
+                    self.charger.configuration = configuration
+                if (
+                    self.aggregate_charger
+                    and self.aggregate_charger.charger_id == self.charger_id
+                ):
+                    self.aggregate_charger.configuration = configuration
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         if action == "TriggerMessage":
-            payload_data = payload if isinstance(payload, dict) else {}
             status_value = str(payload_data.get("status") or "").strip()
             target = metadata.get("trigger_target") or metadata.get("follow_up_action")
             connector_value = metadata.get("trigger_connector")
@@ -653,32 +897,54 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                     log_key=log_key,
                     target=str(target),
                 )
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         if action == "RemoteStartTransaction":
-            payload_data = payload if isinstance(payload, dict) else {}
             status_value = str(payload_data.get("status") or "").strip()
             message = "RemoteStartTransaction result"
             if status_value:
                 message += f": status={status_value}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         if action == "RemoteStopTransaction":
-            payload_data = payload if isinstance(payload, dict) else {}
             status_value = str(payload_data.get("status") or "").strip()
             message = "RemoteStopTransaction result"
             if status_value:
                 message += f": status={status_value}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         if action == "Reset":
-            payload_data = payload if isinstance(payload, dict) else {}
             status_value = str(payload_data.get("status") or "").strip()
             message = "Reset result"
             if status_value:
                 message += f": status={status_value}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         if action != "ChangeAvailability":
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                payload=payload_data,
+            )
             return
         status = str((payload or {}).get("status") or "").strip()
         requested_type = metadata.get("availability_type")
@@ -691,6 +957,11 @@ class CSMSConsumer(AsyncWebsocketConsumer):
             requested_at,
             details="",
         )
+        store.record_pending_call_result(
+            message_id,
+            metadata=metadata,
+            payload=payload_data,
+        )
 
     async def _handle_call_error(
         self,
@@ -709,6 +980,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
         if action == "DataTransfer":
             message_pk = metadata.get("message_pk")
             if not message_pk:
+                store.record_pending_call_result(
+                    message_id,
+                    metadata=metadata,
+                    success=False,
+                    error_code=error_code,
+                    error_description=description,
+                    error_details=details,
+                )
                 return
 
             def _apply():
@@ -735,6 +1014,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 )
 
             await database_sync_to_async(_apply)()
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         if action == "GetConfiguration":
             parts: list[str] = []
@@ -756,6 +1043,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
             else:
                 message = "GetConfiguration error"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         if action == "TriggerMessage":
             target = metadata.get("trigger_target") or metadata.get("follow_up_action")
@@ -780,6 +1075,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
             if connector_value:
                 message += f", connector={connector_value}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         if action == "RemoteStartTransaction":
             message = "RemoteStartTransaction error"
@@ -790,6 +1093,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 if suffix:
                     message += f", description={suffix}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         if action == "RemoteStopTransaction":
             message = "RemoteStopTransaction error"
@@ -800,6 +1111,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 if suffix:
                     message += f", description={suffix}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         if action == "Reset":
             message = "Reset error"
@@ -810,8 +1129,24 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 if suffix:
                     message += f", description={suffix}"
             store.add_log(log_key, message, log_type="charger")
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         if action != "ChangeAvailability":
+            store.record_pending_call_result(
+                message_id,
+                metadata=metadata,
+                success=False,
+                error_code=error_code,
+                error_description=description,
+                error_details=details,
+            )
             return
         detail_text = (description or "").strip()
         if not detail_text and details:
@@ -831,6 +1166,14 @@ class CSMSConsumer(AsyncWebsocketConsumer):
             requested_at,
             details=detail_text,
         )
+        store.record_pending_call_result(
+            message_id,
+            metadata=metadata,
+            success=False,
+            error_code=error_code,
+            error_description=description,
+            error_details=details,
+        )
 
     async def _handle_data_transfer(
         self, message_id: str, payload: dict | None
@@ -1079,8 +1422,13 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 reply_payload = {"currentTime": datetime.utcnow().isoformat() + "Z"}
                 now = timezone.now()
                 self.charger.last_heartbeat = now
+                if (
+                    self.aggregate_charger
+                    and self.aggregate_charger is not self.charger
+                ):
+                    self.aggregate_charger.last_heartbeat = now
                 await database_sync_to_async(
-                    Charger.objects.filter(pk=self.charger.pk).update
+                    Charger.objects.filter(charger_id=self.charger_id).update
                 )(last_heartbeat=now)
             elif action == "StatusNotification":
                 await self._assign_connector(payload.get("connectorId"))
@@ -1133,6 +1481,12 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                     )(**update_kwargs)
                 _update_instance(self.aggregate_charger)
                 _update_instance(self.charger)
+                if connector_value is not None and status.lower() == "available":
+                    tx_obj = store.transactions.pop(self.store_key, None)
+                    if tx_obj:
+                        await self._cancel_consumption_message()
+                        store.end_session_log(self.store_key)
+                        store.stop_session_lock()
                 store.add_log(
                     self.store_key,
                     f"StatusNotification processed: {json.dumps(payload, sort_keys=True)}",
@@ -1145,15 +1499,29 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                     )
                 reply_payload = {}
             elif action == "Authorize":
-                account = await self._get_account(payload.get("idTag"))
+                id_tag = payload.get("idTag")
+                account = await self._get_account(id_tag)
+                status = "Invalid"
                 if self.charger.require_rfid:
-                    status = (
-                        "Accepted"
-                        if account
-                        and await database_sync_to_async(account.can_authorize)()
-                        else "Invalid"
-                    )
+                    tag = None
+                    tag_created = False
+                    if id_tag:
+                        tag, tag_created = await database_sync_to_async(
+                            CoreRFID.register_scan
+                        )(id_tag)
+                    if account:
+                        if await database_sync_to_async(account.can_authorize)():
+                            status = "Accepted"
+                    elif (
+                        id_tag
+                        and tag
+                        and not tag_created
+                        and tag.allowed
+                    ):
+                        status = "Accepted"
+                        self._log_unlinked_rfid(tag.rfid)
                 else:
+                    await self._ensure_rfid_seen(id_tag)
                     status = "Accepted"
                 reply_payload = {"idTagInfo": {"status": status}}
             elif action == "MeterValues":
@@ -1225,28 +1593,51 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                 reply_payload = {}
             elif action == "StartTransaction":
                 id_tag = payload.get("idTag")
-                account = await self._get_account(id_tag)
+                tag = None
+                tag_created = False
                 if id_tag:
-                    await database_sync_to_async(CoreRFID.objects.get_or_create)(
-                        rfid=id_tag.upper()
-                    )
+                    tag, tag_created = await database_sync_to_async(
+                        CoreRFID.register_scan
+                    )(id_tag)
+                account = await self._get_account(id_tag)
+                if id_tag and not self.charger.require_rfid:
+                    seen_tag = await self._ensure_rfid_seen(id_tag)
+                    if seen_tag:
+                        tag = seen_tag
                 await self._assign_connector(payload.get("connectorId"))
+                authorized = True
+                authorized_via_tag = False
                 if self.charger.require_rfid:
-                    authorized = (
-                        account is not None
-                        and await database_sync_to_async(account.can_authorize)()
-                    )
-                else:
-                    authorized = True
+                    if account is not None:
+                        authorized = await database_sync_to_async(
+                            account.can_authorize
+                        )()
+                    elif (
+                        id_tag
+                        and tag
+                        and not tag_created
+                        and getattr(tag, "allowed", False)
+                    ):
+                        authorized = True
+                        authorized_via_tag = True
+                    else:
+                        authorized = False
                 if authorized:
+                    if authorized_via_tag and tag:
+                        self._log_unlinked_rfid(tag.rfid)
+                    start_timestamp = _parse_ocpp_timestamp(payload.get("timestamp"))
+                    received_start = timezone.now()
+                    vid_value, vin_value = _extract_vehicle_identifier(payload)
                     tx_obj = await database_sync_to_async(Transaction.objects.create)(
                         charger=self.charger,
                         account=account,
                         rfid=(id_tag or ""),
-                        vin=(payload.get("vin") or ""),
+                        vid=vid_value,
+                        vin=vin_value,
                         connector_id=payload.get("connectorId"),
                         meter_start=payload.get("meterStart"),
-                        start_time=timezone.now(),
+                        start_time=start_timestamp or received_start,
+                        received_start_time=received_start,
                     )
                     store.transactions[self.store_key] = tx_obj
                     store.start_session_log(self.store_key, tx_obj.pk)
@@ -1267,17 +1658,29 @@ class CSMSConsumer(AsyncWebsocketConsumer):
                         Transaction.objects.filter(pk=tx_id, charger=self.charger).first
                     )()
                 if not tx_obj and tx_id is not None:
+                    received_start = timezone.now()
+                    vid_value, vin_value = _extract_vehicle_identifier(payload)
                     tx_obj = await database_sync_to_async(Transaction.objects.create)(
                         pk=tx_id,
                         charger=self.charger,
-                        start_time=timezone.now(),
+                        start_time=received_start,
+                        received_start_time=received_start,
                         meter_start=payload.get("meterStart")
                         or payload.get("meterStop"),
-                        vin=(payload.get("vin") or ""),
+                        vid=vid_value,
+                        vin=vin_value,
                     )
                 if tx_obj:
+                    stop_timestamp = _parse_ocpp_timestamp(payload.get("timestamp"))
+                    received_stop = timezone.now()
                     tx_obj.meter_stop = payload.get("meterStop")
-                    tx_obj.stop_time = timezone.now()
+                    vid_value, vin_value = _extract_vehicle_identifier(payload)
+                    if vid_value:
+                        tx_obj.vid = vid_value
+                    if vin_value:
+                        tx_obj.vin = vin_value
+                    tx_obj.stop_time = stop_timestamp or received_stop
+                    tx_obj.received_stop_time = received_stop
                     await database_sync_to_async(tx_obj.save)()
                     await self._update_consumption_message(tx_obj.pk)
                 await self._cancel_consumption_message()
