@@ -186,6 +186,8 @@ async def simulate_cp(
     interval: float = 5.0,
     username: Optional[str] = None,
     password: Optional[str] = None,
+    *,
+    sim_state: SimulatorState | None = None,
 ) -> None:
     """Simulate one charge point session.
 
@@ -206,7 +208,7 @@ async def simulate_cp(
         b64 = base64.b64encode(userpass.encode("utf-8")).decode("ascii")
         headers["Authorization"] = f"Basic {b64}"
 
-    state = _simulators.get(cp_idx + 1, _simulators[1])
+    state = sim_state or _simulators.get(cp_idx + 1, _simulators[1])
 
     loop_count = 0
     while loop_count < session_count and state.running:
@@ -247,99 +249,6 @@ async def simulate_cp(
             stop_event = asyncio.Event()
             reset_event = asyncio.Event()
 
-            async def listen():
-                try:
-                    while True:
-                        raw = await _recv()
-                        try:
-                            msg = json.loads(raw)
-                        except json.JSONDecodeError:
-                            continue
-
-                        if isinstance(msg, list) and msg and msg[0] == 2:
-                            msg_id, action = msg[1], msg[2]
-                            await _send([3, msg_id, {}])
-                            if action == "RemoteStopTransaction":
-                                state.last_message = "RemoteStopTransaction"
-                                stop_event.set()
-                            elif action == "Reset":
-                                state.last_message = "Reset"
-                                reset_event.set()
-                                stop_event.set()
-                except websockets.ConnectionClosed:
-                    stop_event.set()
-
-                # boot notification / authorise
-                await _send(
-                    [
-                        2,
-                        "boot",
-                        "BootNotification",
-                        {
-                            "chargePointModel": "Simulator",
-                            "chargePointVendor": "SimVendor",
-                            "serialNumber": serial_number,
-                        },
-                    ]
-                )
-                state.last_message = "BootNotification"
-                await _recv()
-                await _send([2, "auth", "Authorize", {"idTag": rfid}])
-                state.last_message = "Authorize"
-                await _recv()
-
-                state.phase = "Available"
-
-                meter_start = random.randint(1000, 2000)
-                actual_duration = random.uniform(duration * 0.75, duration * 1.25)
-                steps = max(1, int(actual_duration / interval))
-                step_min = max(1, int((kw_min * 1000) / steps))
-                step_max = max(1, int((kw_max * 1000) / steps))
-
-                # optional pre‑charge delay while still sending heartbeats
-                if pre_charge_delay > 0:
-                    start_delay = time.monotonic()
-                    next_meter = meter_start
-                    last_mv = time.monotonic()
-                    while time.monotonic() - start_delay < pre_charge_delay:
-                        await _send([2, "hb", "Heartbeat", {}])
-                        state.last_message = "Heartbeat"
-                        await _recv()
-                        await asyncio.sleep(5)
-                        if time.monotonic() - last_mv >= 30:
-                            idle_step = max(2, int(step_max / 100))
-                            next_meter += random.randint(0, idle_step)
-                            next_kw = next_meter / 1000.0
-                            await _send(
-                                [
-                                    2,
-                                    "meter",
-                                    "MeterValues",
-                                    {
-                                        "connectorId": connector_id,
-                                        "meterValue": [
-                                            {
-                                                "timestamp": time.strftime(
-                                                    "%Y-%m-%dT%H:%M:%S"
-                                                )
-                                                + "Z",
-                                                "sampledValue": [
-                                                    {
-                                                        "value": f"{next_kw:.3f}",
-                                                        "measurand": "Energy.Active.Import.Register",
-                                                        "unit": "kW",
-                                                        "context": "Sample.Clock",
-                                                    }
-                                                ],
-                                            }
-                                        ],
-                                    },
-                                ]
-                            )
-                            state.last_message = "MeterValues"
-                            await _recv()
-                            last_mv = time.monotonic()
-
             async def listen():
                 try:
                     while True:
@@ -663,6 +572,7 @@ def simulate(
                 interval,
                 username,
                 password,
+                sim_state=state,
             )
 
         def run_thread(idx: int) -> None:
@@ -685,6 +595,7 @@ def simulate(
                     interval,
                     username,
                     password,
+                    sim_state=state,
                 )
             )
 
@@ -737,6 +648,7 @@ def simulate(
                 interval,
                 username,
                 password,
+                sim_state=state,
             )
         )
     else:
@@ -764,6 +676,7 @@ def simulate(
                     username,
                     password,
                 ),
+                kwargs={"sim_state": state},
                 daemon=True,
             )
             t.start()
