@@ -1,8 +1,10 @@
 import io
 import json
 import os
+import subprocess
 import sys
 import types
+from datetime import datetime, timezone as dt_timezone
 from pathlib import Path
 from unittest.mock import patch, MagicMock, call
 
@@ -26,7 +28,7 @@ from nodes.models import Node, NodeRole
 from core.models import RFID
 from ocpp.rfid.reader import read_rfid, enable_deep_read, validate_rfid_value
 from ocpp.rfid.detect import detect_scanner, main as detect_main
-from ocpp.rfid import background_reader
+from ocpp.rfid import background_reader, camera
 from ocpp.rfid.constants import (
     DEFAULT_IRQ_PIN,
     DEFAULT_RST_PIN,
@@ -129,7 +131,7 @@ class ScanNextViewTests(TestCase):
         self.assertEqual(
             resp.json(), {"rfid": "ABCD1234", "label_id": 1, "created": False}
         )
-        mock_validate.assert_called_once_with("ABCD1234", kind=None)
+        mock_validate.assert_called_once_with("ABCD1234", kind=None, endianness=None)
 
     @patch("config.middleware.Node.get_local", return_value=None)
     @patch("config.middleware.get_site")
@@ -137,6 +139,7 @@ class ScanNextViewTests(TestCase):
     def test_scan_next_post_requires_authentication(
         self, mock_validate, mock_site, mock_node
     ):
+        self.client.logout()
         resp = self.client.post(
             reverse("rfid-scan-next"),
             data=json.dumps({"rfid": "ABCD1234"}),
@@ -168,6 +171,15 @@ class ScanNextViewTests(TestCase):
 
 
 class ReaderNotificationTests(TestCase):
+    def setUp(self):
+        super().setUp()
+        self.queue_patcher = patch("ocpp.rfid.reader.queue_camera_snapshot")
+        self.mock_queue = self.queue_patcher.start()
+
+    def tearDown(self):
+        self.queue_patcher.stop()
+        super().tearDown()
+
     def _mock_reader(self):
         class MockReader:
             MI_OK = 1
@@ -189,8 +201,8 @@ class ReaderNotificationTests(TestCase):
         return MockReader()
 
     @patch("ocpp.rfid.reader.notify_async")
-    @patch("core.models.RFID.objects.get_or_create")
-    def test_notify_on_allowed_tag(self, mock_get, mock_notify):
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_notify_on_allowed_tag(self, mock_register, mock_notify):
         reference = MagicMock(value="https://example.com")
         tag = MagicMock(
             label_id=1,
@@ -200,7 +212,7 @@ class ReaderNotificationTests(TestCase):
             released=False,
             reference=reference,
         )
-        mock_get.return_value = (tag, False)
+        mock_register.return_value = (tag, False)
 
         reader = self._mock_reader()
         result = read_rfid(mfrc=reader, cleanup=False)
@@ -213,8 +225,8 @@ class ReaderNotificationTests(TestCase):
         self.assertTrue(getattr(reader, "stop_called", False))
 
     @patch("ocpp.rfid.reader.notify_async")
-    @patch("core.models.RFID.objects.get_or_create")
-    def test_notify_on_disallowed_tag(self, mock_get, mock_notify):
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_notify_on_disallowed_tag(self, mock_register, mock_notify):
         tag = MagicMock(
             label_id=2,
             pk=2,
@@ -223,7 +235,7 @@ class ReaderNotificationTests(TestCase):
             released=False,
             reference=None,
         )
-        mock_get.return_value = (tag, False)
+        mock_register.return_value = (tag, False)
 
         reader = self._mock_reader()
         result = read_rfid(mfrc=reader, cleanup=False)
@@ -233,12 +245,93 @@ class ReaderNotificationTests(TestCase):
         self.assertTrue(getattr(reader, "select_called", False))
         self.assertTrue(getattr(reader, "stop_called", False))
 
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_snapshot_metadata_passed_to_queue(self, mock_register, mock_notify):
+        tag = MagicMock(
+            label_id=5,
+            pk=5,
+            allowed=True,
+            color="",
+            released=False,
+            reference=None,
+        )
+        mock_register.return_value = (tag, True)
+
+        reader = self._mock_reader()
+        result = read_rfid(mfrc=reader, cleanup=False)
+
+        self.assertTrue(self.mock_queue.called)
+        args, kwargs = self.mock_queue.call_args
+        self.assertEqual(args[0], result["rfid"])
+        self.assertEqual(args[1]["label_id"], tag.pk)
+        self.assertTrue(args[1]["created"])
+
+
+class CameraSnapshotTriggerTests(SimpleTestCase):
+    @patch("ocpp.rfid.camera._camera_feature_enabled", return_value=False)
+    @patch("ocpp.rfid.camera.threading.Thread")
+    def test_queue_skips_without_feature(self, mock_thread, mock_enabled):
+        camera.queue_camera_snapshot("ABC123", {"label_id": 1})
+        mock_thread.assert_not_called()
+
+    @patch("ocpp.rfid.camera.timezone.now")
+    @patch("ocpp.rfid.camera.threading.Thread")
+    @patch("ocpp.rfid.camera._camera_feature_enabled", return_value=True)
+    def test_queue_spawns_thread_with_metadata(
+        self, mock_enabled, mock_thread, mock_now
+    ):
+        thread_instance = MagicMock()
+        mock_thread.return_value = thread_instance
+        mock_now.return_value = datetime(2023, 1, 1, tzinfo=dt_timezone.utc)
+
+        camera.queue_camera_snapshot("ABC123", {"label_id": 7})
+
+        mock_thread.assert_called_once()
+        kwargs = mock_thread.call_args.kwargs
+        self.assertTrue(kwargs.get("daemon"))
+        metadata = kwargs["args"][0]
+        self.assertEqual(metadata["rfid"], "ABC123")
+        self.assertEqual(metadata["label_id"], 7)
+        self.assertEqual(metadata["source"], "rfid-scan")
+        self.assertEqual(metadata["captured_at"], "2023-01-01T00:00:00+00:00")
+        thread_instance.start.assert_called_once()
+
+    @patch("ocpp.rfid.camera.close_old_connections")
+    @patch("ocpp.rfid.camera.save_screenshot")
+    @patch("ocpp.rfid.camera.capture_rpi_snapshot")
+    def test_worker_saves_snapshot(
+        self, mock_capture, mock_save, mock_close
+    ):
+        mock_capture.return_value = Path("/tmp/test.jpg")
+
+        camera._capture_snapshot_worker({"rfid": "ABC", "label_id": 3})
+
+        mock_capture.assert_called_once()
+        mock_save.assert_called_once()
+        _, kwargs = mock_save.call_args
+        self.assertEqual(kwargs["method"], "RFID_SCAN")
+        metadata = json.loads(kwargs["content"])
+        self.assertEqual(metadata["rfid"], "ABC")
+        self.assertEqual(metadata["label_id"], 3)
+        self.assertGreaterEqual(mock_close.call_count, 2)
+
+    @patch("ocpp.rfid.camera.close_old_connections")
+    @patch("ocpp.rfid.camera.save_screenshot")
+    @patch("ocpp.rfid.camera.capture_rpi_snapshot", side_effect=RuntimeError("boom"))
+    def test_worker_handles_capture_failure(
+        self, mock_capture, mock_save, mock_close
+    ):
+        camera._capture_snapshot_worker({"rfid": "XYZ"})
+        mock_save.assert_not_called()
+        self.assertGreaterEqual(mock_close.call_count, 2)
+
 
 class ValidateRfidValueTests(SimpleTestCase):
     @patch("ocpp.rfid.reader.timezone.now")
     @patch("ocpp.rfid.reader.notify_async")
-    @patch("ocpp.rfid.reader.RFID.objects.get_or_create")
-    def test_creates_new_tag(self, mock_get, mock_notify, mock_now):
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_creates_new_tag(self, mock_register, mock_notify, mock_now):
         fake_now = object()
         mock_now.return_value = fake_now
         tag = MagicMock()
@@ -249,21 +342,25 @@ class ValidateRfidValueTests(SimpleTestCase):
         tag.released = False
         tag.reference = None
         tag.kind = RFID.CLASSIC
-        mock_get.return_value = (tag, True)
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, True)
 
         result = validate_rfid_value("abcd1234")
 
-        mock_get.assert_called_once_with(rfid="ABCD1234", defaults={})
+        mock_register.assert_called_once_with(
+            "ABCD1234", kind=None, endianness=RFID.BIG_ENDIAN
+        )
         tag.save.assert_called_once_with(update_fields=["last_seen_on"])
         self.assertIs(tag.last_seen_on, fake_now)
         mock_notify.assert_called_once_with("RFID 1 OK", "ABCD1234 B")
         self.assertTrue(result["created"])
         self.assertEqual(result["rfid"], "ABCD1234")
+        self.assertEqual(result["endianness"], RFID.BIG_ENDIAN)
 
     @patch("ocpp.rfid.reader.timezone.now")
     @patch("ocpp.rfid.reader.notify_async")
-    @patch("ocpp.rfid.reader.RFID.objects.get_or_create")
-    def test_updates_existing_tag_kind(self, mock_get, mock_notify, mock_now):
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_updates_existing_tag_kind(self, mock_register, mock_notify, mock_now):
         fake_now = object()
         mock_now.return_value = fake_now
         tag = MagicMock()
@@ -274,12 +371,13 @@ class ValidateRfidValueTests(SimpleTestCase):
         tag.released = True
         tag.reference = None
         tag.kind = RFID.CLASSIC
-        mock_get.return_value = (tag, False)
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, False)
 
         result = validate_rfid_value("abcd", kind=RFID.NTAG215)
 
-        mock_get.assert_called_once_with(
-            rfid="ABCD", defaults={"kind": RFID.NTAG215}
+        mock_register.assert_called_once_with(
+            "ABCD", kind=RFID.NTAG215, endianness=RFID.BIG_ENDIAN
         )
         tag.save.assert_called_once_with(update_fields=["kind", "last_seen_on"])
         self.assertIs(tag.last_seen_on, fake_now)
@@ -288,6 +386,36 @@ class ValidateRfidValueTests(SimpleTestCase):
         self.assertFalse(result["allowed"])
         self.assertFalse(result["created"])
         self.assertEqual(result["kind"], RFID.NTAG215)
+        self.assertEqual(result["endianness"], RFID.BIG_ENDIAN)
+
+    @patch("ocpp.rfid.reader.timezone.now")
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_registers_little_endian_value(
+        self, mock_register, mock_notify, mock_now
+    ):
+        fake_now = object()
+        mock_now.return_value = fake_now
+        tag = MagicMock()
+        tag.pk = 7
+        tag.label_id = 7
+        tag.allowed = True
+        tag.color = "B"
+        tag.released = False
+        tag.reference = None
+        tag.kind = RFID.CLASSIC
+        tag.endianness = RFID.LITTLE_ENDIAN
+        mock_register.return_value = (tag, True)
+
+        result = validate_rfid_value("A1B2C3D4", endianness=RFID.LITTLE_ENDIAN)
+
+        mock_register.assert_called_once_with(
+            "D4C3B2A1", kind=None, endianness=RFID.LITTLE_ENDIAN
+        )
+        tag.save.assert_called_once_with(update_fields=["last_seen_on"])
+        self.assertEqual(result["rfid"], "D4C3B2A1")
+        self.assertEqual(result["endianness"], RFID.LITTLE_ENDIAN)
+        mock_notify.assert_called_once()
 
     def test_rejects_invalid_value(self):
         result = validate_rfid_value("invalid!")
@@ -302,6 +430,204 @@ class ValidateRfidValueTests(SimpleTestCase):
         self.assertEqual(result, {"error": "RFID value is required"})
 
 
+    @patch("ocpp.rfid.reader.timezone.now")
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.subprocess.Popen")
+    @patch("ocpp.rfid.reader.subprocess.run")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_external_command_success(
+        self, mock_register, mock_run, mock_popen, mock_notify, mock_now
+    ):
+        fake_now = object()
+        mock_now.return_value = fake_now
+        tag = MagicMock()
+        tag.pk = 1
+        tag.label_id = 1
+        tag.allowed = True
+        tag.external_command = "echo ok"
+        tag.color = "B"
+        tag.released = False
+        tag.reference = None
+        tag.kind = RFID.CLASSIC
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, False)
+        mock_run.return_value = types.SimpleNamespace(
+            returncode=0, stdout="ok\n", stderr=""
+        )
+        mock_popen.return_value = object()
+
+        result = validate_rfid_value("abcd1234")
+
+        mock_run.assert_called_once()
+        run_args, run_kwargs = mock_run.call_args
+        self.assertEqual(run_args[0], "echo ok")
+        self.assertTrue(run_kwargs.get("shell"))
+        env = run_kwargs.get("env", {})
+        self.assertEqual(env.get("RFID_VALUE"), "ABCD1234")
+        self.assertEqual(env.get("RFID_LABEL_ID"), "1")
+        self.assertEqual(env.get("RFID_ENDIANNESS"), RFID.BIG_ENDIAN)
+        mock_popen.assert_not_called()
+        mock_notify.assert_called_once_with("RFID 1 OK", "ABCD1234 B")
+        tag.save.assert_called_once_with(update_fields=["last_seen_on"])
+        self.assertTrue(result["allowed"])
+        output = result.get("command_output")
+        self.assertIsInstance(output, dict)
+        self.assertEqual(output.get("stdout"), "ok\n")
+        self.assertEqual(output.get("stderr"), "")
+        self.assertEqual(output.get("returncode"), 0)
+        self.assertEqual(output.get("error"), "")
+        self.assertEqual(result["endianness"], RFID.BIG_ENDIAN)
+
+    @patch("ocpp.rfid.reader.timezone.now")
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.subprocess.Popen")
+    @patch("ocpp.rfid.reader.subprocess.run")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_external_command_failure_blocks_tag(
+        self, mock_register, mock_run, mock_popen, mock_notify, mock_now
+    ):
+        fake_now = object()
+        mock_now.return_value = fake_now
+        tag = MagicMock()
+        tag.pk = 2
+        tag.label_id = 2
+        tag.allowed = True
+        tag.external_command = "exit 1"
+        tag.color = "G"
+        tag.released = False
+        tag.reference = None
+        tag.kind = RFID.CLASSIC
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, False)
+        mock_run.return_value = types.SimpleNamespace(
+            returncode=1, stdout="", stderr="failure"
+        )
+        mock_popen.return_value = object()
+
+        result = validate_rfid_value("ffff")
+
+        mock_run.assert_called_once()
+        mock_notify.assert_called_once_with("RFID 2 BAD", "FFFF G")
+        tag.save.assert_called_once_with(update_fields=["last_seen_on"])
+        self.assertFalse(result["allowed"])
+        output = result.get("command_output")
+        self.assertIsInstance(output, dict)
+        self.assertEqual(output.get("returncode"), 1)
+        self.assertEqual(output.get("stdout"), "")
+        self.assertEqual(output.get("stderr"), "failure")
+        self.assertEqual(output.get("error"), "")
+        mock_popen.assert_not_called()
+        self.assertEqual(result["endianness"], RFID.BIG_ENDIAN)
+
+    @patch("ocpp.rfid.reader.timezone.now")
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.subprocess.Popen")
+    @patch("ocpp.rfid.reader.subprocess.run")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_external_command_strips_trailing_percent_tokens(
+        self, mock_register, mock_run, mock_popen, mock_notify, mock_now
+    ):
+        mock_now.return_value = timezone.now()
+        tag = MagicMock()
+        tag.pk = 3
+        tag.label_id = 3
+        tag.allowed = True
+        tag.external_command = "echo weird"
+        tag.color = "Y"
+        tag.released = False
+        tag.reference = None
+        tag.kind = RFID.CLASSIC
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, False)
+        mock_run.return_value = types.SimpleNamespace(
+            returncode=0,
+            stdout="first %\nsecond 50%\r\nthird % %\n",
+            stderr="oops %\n",
+        )
+
+        result = validate_rfid_value("abc3")
+
+        output = result.get("command_output")
+        self.assertIsNotNone(output)
+        self.assertEqual(
+            output.get("stdout"), "first\nsecond 50%\r\nthird\n"
+        )
+        self.assertEqual(output.get("stderr"), "oops\n")
+        self.assertEqual(output.get("returncode"), 0)
+        self.assertEqual(output.get("error"), "")
+        mock_popen.assert_not_called()
+
+    @patch("ocpp.rfid.reader.timezone.now")
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.subprocess.Popen")
+    @patch("ocpp.rfid.reader.subprocess.run")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_external_command_error_strips_trailing_percent_tokens(
+        self, mock_register, mock_run, mock_popen, mock_notify, mock_now
+    ):
+        mock_now.return_value = timezone.now()
+        tag = MagicMock()
+        tag.pk = 4
+        tag.label_id = 4
+        tag.allowed = True
+        tag.external_command = "echo boom"
+        tag.color = "R"
+        tag.released = False
+        tag.reference = None
+        tag.kind = RFID.CLASSIC
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, False)
+        mock_run.side_effect = RuntimeError("bad % %")
+
+        result = validate_rfid_value("abcd")
+
+        output = result.get("command_output")
+        self.assertIsInstance(output, dict)
+        self.assertEqual(output.get("stdout"), "")
+        self.assertEqual(output.get("stderr"), "")
+        self.assertEqual(output.get("error"), "bad")
+        self.assertFalse(result["allowed"])
+        mock_popen.assert_not_called()
+
+    @patch("ocpp.rfid.reader.timezone.now")
+    @patch("ocpp.rfid.reader.notify_async")
+    @patch("ocpp.rfid.reader.subprocess.Popen")
+    @patch("ocpp.rfid.reader.subprocess.run")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_post_command_runs_after_success(
+        self, mock_register, mock_run, mock_popen, mock_notify, mock_now
+    ):
+        fake_now = object()
+        mock_now.return_value = fake_now
+        tag = MagicMock()
+        tag.pk = 3
+        tag.label_id = 3
+        tag.allowed = True
+        tag.external_command = ""
+        tag.post_auth_command = "echo done"
+        tag.color = "B"
+        tag.released = False
+        tag.reference = None
+        tag.kind = RFID.CLASSIC
+        tag.endianness = RFID.BIG_ENDIAN
+        mock_register.return_value = (tag, False)
+        result = validate_rfid_value("abcdef")
+
+        mock_run.assert_not_called()
+        mock_popen.assert_called_once()
+        args, kwargs = mock_popen.call_args
+        self.assertEqual(args[0], "echo done")
+        env = kwargs.get("env", {})
+        self.assertEqual(env.get("RFID_VALUE"), "ABCDEF")
+        self.assertEqual(env.get("RFID_LABEL_ID"), "3")
+        self.assertEqual(env.get("RFID_ENDIANNESS"), RFID.BIG_ENDIAN)
+        self.assertIs(kwargs.get("stdout"), subprocess.DEVNULL)
+        self.assertIs(kwargs.get("stderr"), subprocess.DEVNULL)
+        self.assertTrue(result["allowed"])
+        mock_notify.assert_called_once_with("RFID 3 OK", "ABCDEF B")
+        self.assertEqual(result["endianness"], RFID.BIG_ENDIAN)
+
+
 class CardTypeDetectionTests(TestCase):
     def _mock_ntag_reader(self):
         class MockReader:
@@ -327,8 +653,8 @@ class CardTypeDetectionTests(TestCase):
         return MockReader()
 
     @patch("ocpp.rfid.reader.notify_async")
-    @patch("core.models.RFID.objects.get_or_create")
-    def test_detects_ntag215(self, mock_get, _mock_notify):
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_detects_ntag215(self, mock_register, _mock_notify):
         tag = MagicMock(
             pk=1,
             label_id=1,
@@ -338,7 +664,7 @@ class CardTypeDetectionTests(TestCase):
             reference=None,
             kind=RFID.NTAG215,
         )
-        mock_get.return_value = (tag, True)
+        mock_register.return_value = (tag, True)
         reader = self._mock_ntag_reader()
         result = read_rfid(mfrc=reader, cleanup=False)
         self.assertEqual(result["kind"], RFID.NTAG215)
@@ -432,60 +758,6 @@ class RFIDDetectionScriptTests(SimpleTestCase):
         mock_detect.assert_called_once()
 
 
-class RestartViewTests(TestCase):
-    def setUp(self):
-        User = get_user_model()
-        self.user = User.objects.create_user("restart-user", password="pwd")
-        self.client.force_login(self.user)
-
-    @patch("config.middleware.Node.get_local", return_value=None)
-    @patch("config.middleware.get_site")
-    @patch("ocpp.rfid.views.restart_sources", return_value={"status": "restarted"})
-    def test_restart_endpoint(self, mock_restart, mock_site, mock_node):
-        resp = self.client.post(reverse("rfid-scan-restart"))
-        self.assertEqual(resp.status_code, 200)
-        self.assertEqual(resp.json(), {"status": "restarted"})
-        mock_restart.assert_called_once()
-
-    def test_restart_requires_authentication(self):
-        self.client.logout()
-        resp = self.client.post(reverse("rfid-scan-restart"))
-        self.assertEqual(resp.status_code, 302)
-        self.assertIn(reverse("pages:login"), resp.url)
-
-
-class ScanTestViewTests(TestCase):
-    def setUp(self):
-        User = get_user_model()
-        self.user = User.objects.create_user("scan-test-user", password="pwd")
-        self.client.force_login(self.user)
-
-    @patch("config.middleware.Node.get_local", return_value=None)
-    @patch("config.middleware.get_site")
-    @patch("ocpp.rfid.views.test_sources", return_value={"irq_pin": 7})
-    def test_scan_test_success(self, mock_test, mock_site, mock_node):
-        resp = self.client.get(reverse("rfid-scan-test"))
-        self.assertEqual(resp.status_code, 200)
-        self.assertEqual(resp.json(), {"irq_pin": 7})
-
-    @patch("config.middleware.Node.get_local", return_value=None)
-    @patch("config.middleware.get_site")
-    @patch(
-        "ocpp.rfid.views.test_sources",
-        return_value={"error": "no scanner detected"},
-    )
-    def test_scan_test_error(self, mock_test, mock_site, mock_node):
-        resp = self.client.get(reverse("rfid-scan-test"))
-        self.assertEqual(resp.status_code, 500)
-        self.assertEqual(resp.json(), {"error": "no scanner detected"})
-
-    def test_scan_test_requires_authentication(self):
-        self.client.logout()
-        resp = self.client.get(reverse("rfid-scan-test"))
-        self.assertEqual(resp.status_code, 302)
-        self.assertIn(reverse("pages:login"), resp.url)
-
-
 class RFIDLandingTests(TestCase):
     def test_scanner_view_registered_as_landing(self):
         role, _ = NodeRole.objects.get_or_create(name="Terminal")
@@ -499,7 +771,9 @@ class RFIDLandingTests(TestCase):
         app = Application.objects.create(name="Ocpp")
         module = Module.objects.create(node_role=role, application=app, path="/ocpp/")
         module.create_landings()
-        self.assertTrue(module.landings.filter(path="/ocpp/rfid/").exists())
+        self.assertTrue(
+            module.landings.filter(path="/ocpp/rfid/validator/").exists()
+        )
 
 
 class ScannerTemplateTests(TestCase):
@@ -514,6 +788,8 @@ class ScannerTemplateTests(TestCase):
         self.client.force_login(staff)
         resp = self.client.get(self.url)
         self.assertContains(resp, 'id="rfid-configure"')
+        self.assertContains(resp, 'id="rfid-connect-local"')
+        self.assertNotContains(resp, 'Restart &amp; Test Scanner')
 
     def test_redirect_for_anonymous(self):
         self.client.logout()
@@ -530,6 +806,7 @@ class ScannerTemplateTests(TestCase):
         self.assertContains(resp, 'id="rfid-rfid"')
         self.assertContains(resp, 'id="rfid-released"')
         self.assertContains(resp, 'id="rfid-reference"')
+        self.assertContains(resp, 'id="rfid-deep-details"')
 
     def test_basic_fields_for_authenticated_user(self):
         self.client.logout()
@@ -540,6 +817,8 @@ class ScannerTemplateTests(TestCase):
         self.assertNotContains(resp, 'id="rfid-rfid"')
         self.assertNotContains(resp, 'id="rfid-released"')
         self.assertNotContains(resp, 'id="rfid-reference"')
+        self.assertNotContains(resp, 'id="rfid-deep-details"')
+        self.assertNotContains(resp, 'Restart &amp; Test Scanner')
 
     def test_deep_read_button_for_staff(self):
         User = get_user_model()
@@ -592,7 +871,7 @@ class DeepReadViewTests(TestCase):
     @patch("config.middleware.get_site")
     @patch(
         "ocpp.rfid.views.enable_deep_read_mode",
-        return_value={"status": "deep", "timeout": 60},
+        return_value={"status": "deep read enabled", "enabled": True},
     )
     def test_enable_deep_read(self, mock_enable, mock_site, mock_node):
         User = get_user_model()
@@ -600,7 +879,9 @@ class DeepReadViewTests(TestCase):
         self.client.force_login(staff)
         resp = self.client.post(reverse("rfid-scan-deep"))
         self.assertEqual(resp.status_code, 200)
-        self.assertEqual(resp.json(), {"status": "deep", "timeout": 60})
+        self.assertEqual(
+            resp.json(), {"status": "deep read enabled", "enabled": True}
+        )
         mock_enable.assert_called_once()
 
     def test_forbidden_for_anonymous(self):
@@ -609,6 +890,15 @@ class DeepReadViewTests(TestCase):
 
 
 class DeepReadAuthTests(TestCase):
+    def setUp(self):
+        super().setUp()
+        self.queue_patcher = patch("ocpp.rfid.reader.queue_camera_snapshot")
+        self.queue_patcher.start()
+
+    def tearDown(self):
+        self.queue_patcher.stop()
+        super().tearDown()
+
     class MockReader:
         MI_OK = 1
         MI_ERR = 2
@@ -633,8 +923,8 @@ class DeepReadAuthTests(TestCase):
             return (self.MI_OK, [0] * 16)
 
     @patch("core.notifications.notify_async")
-    @patch("core.models.RFID.objects.get_or_create")
-    def test_auth_tries_key_a_then_b(self, mock_get, mock_notify):
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_auth_tries_key_a_then_b(self, mock_register, mock_notify):
         tag = MagicMock(
             label_id=1,
             pk=1,
@@ -643,13 +933,93 @@ class DeepReadAuthTests(TestCase):
             released=False,
             reference=None,
         )
-        mock_get.return_value = (tag, False)
+        tag.key_a = "A1A2A3A4A5A6"
+        tag.key_b = "B1B2B3B4B5B6"
+        tag.key_a_verified = True
+        tag.key_b_verified = False
+        tag.data = []
+        tag.save = MagicMock()
+        mock_register.return_value = (tag, False)
         reader = self.MockReader()
         enable_deep_read(60)
-        read_rfid(mfrc=reader, cleanup=False)
+        result = read_rfid(mfrc=reader, cleanup=False)
         self.assertGreaterEqual(len(reader.auth_calls), 2)
         self.assertEqual(reader.auth_calls[0], reader.PICC_AUTHENT1A)
         self.assertEqual(reader.auth_calls[1], reader.PICC_AUTHENT1B)
+        self.assertTrue(result.get("deep_read"))
+        self.assertIn("dump", result)
+        self.assertIn("keys", result)
+        self.assertEqual(result["keys"].get("a"), "A1A2A3A4A5A6")
+        self.assertTrue(result["keys"].get("b_verified"))
+        self.assertTrue(tag.key_b_verified)
+        self.assertTrue(any(entry.get("key") == "B" for entry in result["dump"]))
+        self.assertEqual(tag.data, result["dump"])
+        self.assertTrue(
+            any(
+                "data" in set(kwargs.get("update_fields", []) or [])
+                for _args, kwargs in tag.save.call_args_list
+            )
+        )
+
+    @patch("core.notifications.notify_async")
+    @patch("ocpp.rfid.reader.RFID.register_scan")
+    def test_heuristic_verifies_unverified_keys(
+        self, mock_register, mock_notify
+    ):
+        tag = MagicMock(
+            label_id=1,
+            pk=1,
+            allowed=True,
+            color="B",
+            released=False,
+            reference=None,
+        )
+        tag.key_a = "111111111111"
+        tag.key_b = "FFFFFFFFFFFF"
+        tag.key_a_verified = False
+        tag.key_b_verified = True
+        tag.save = MagicMock()
+        mock_register.return_value = (tag, False)
+
+        class HeuristicReader:
+            MI_OK = 1
+            MI_ERR = 2
+            PICC_REQIDL = 0
+            PICC_AUTHENT1A = 0x60
+            PICC_AUTHENT1B = 0x61
+
+            def __init__(self):
+                self.auth_calls: list[tuple[int, str]] = []
+
+            def MFRC522_Request(self, _):
+                return (self.MI_OK, None)
+
+            def MFRC522_Anticoll(self):
+                return (self.MI_OK, [0x01, 0x02, 0x03, 0x04])
+
+            def MFRC522_Auth(self, mode, block, key, uid):
+                key_hex = "".join(f"{value:02X}" for value in key)
+                self.auth_calls.append((mode, key_hex))
+                if mode == self.PICC_AUTHENT1A and key_hex == "A0A1A2A3A4A5":
+                    return self.MI_OK
+                return self.MI_ERR
+
+            def MFRC522_Read(self, block):
+                return (self.MI_OK, list(range(16)))
+
+        reader = HeuristicReader()
+        enable_deep_read(60)
+        result = read_rfid(mfrc=reader, cleanup=False)
+
+        self.assertIn((reader.PICC_AUTHENT1A, "A0A1A2A3A4A5"), reader.auth_calls)
+        self.assertEqual(result["keys"].get("a"), "A0A1A2A3A4A5")
+        self.assertTrue(result["keys"].get("a_verified"))
+        self.assertEqual(tag.key_a, "A0A1A2A3A4A5")
+        self.assertTrue(tag.key_a_verified)
+        self.assertIn(
+            call(update_fields=["key_a", "key_a_verified"]),
+            tag.save.call_args_list,
+        )
 
 
 class RFIDWiringConfigTests(SimpleTestCase):
