@@ -2,28 +2,35 @@ from utils.sites import get_site
 from django.urls import Resolver404, resolve
 from django.conf import settings
 from pathlib import Path
-from types import SimpleNamespace
 from nodes.models import Node
 from core.models import Reference
 from core.reference_utils import filter_visible_references
 from .models import Module
 
-_favicon_path = Path(settings.BASE_DIR) / "pages" / "fixtures" / "data" / "favicon.txt"
-_control_favicon_path = (
-    Path(settings.BASE_DIR) / "pages" / "fixtures" / "data" / "favicon_control.txt"
-)
+_FAVICON_DIR = Path(settings.BASE_DIR) / "pages" / "fixtures" / "data"
+_FAVICON_FILENAMES = {
+    "default": "favicon.txt",
+    "Watchtower": "favicon_watchtower.txt",
+    "Constellation": "favicon_watchtower.txt",
+    "Control": "favicon_control.txt",
+    "Satellite": "favicon_satellite.txt",
+}
 
-try:
-    _DEFAULT_FAVICON = f"data:image/png;base64,{_favicon_path.read_text().strip()}"
-except OSError:
-    _DEFAULT_FAVICON = ""
 
-try:
-    _CONTROL_FAVICON = (
-        f"data:image/png;base64,{_control_favicon_path.read_text().strip()}"
-    )
-except OSError:
-    _CONTROL_FAVICON = _DEFAULT_FAVICON
+def _load_favicon(filename: str) -> str:
+    path = _FAVICON_DIR / filename
+    try:
+        return f"data:image/png;base64,{path.read_text().strip()}"
+    except OSError:
+        return ""
+
+
+_DEFAULT_FAVICON = _load_favicon(_FAVICON_FILENAMES["default"])
+_ROLE_FAVICONS = {
+    role: (_load_favicon(filename) or _DEFAULT_FAVICON)
+    for role, filename in _FAVICON_FILENAMES.items()
+    if role != "default"
+}
 
 
 def nav_links(request):
@@ -42,6 +49,13 @@ def nav_links(request):
 
     valid_modules = []
     current_module = None
+    user = getattr(request, "user", None)
+    user_is_authenticated = getattr(user, "is_authenticated", False)
+    user_is_superuser = getattr(user, "is_superuser", False)
+    if user_is_authenticated:
+        user_group_names = set(user.groups.values_list("name", flat=True))
+    else:
+        user_group_names = set()
     for module in modules:
         landings = []
         for landing in module.landings.filter(enabled=True):
@@ -54,15 +68,43 @@ def nav_links(request):
             if not requires_login and hasattr(view_func, "login_url"):
                 requires_login = True
             staff_only = getattr(view_func, "staff_required", False)
-            if requires_login and not request.user.is_authenticated:
-                setattr(landing, "requires_login", True)
-            if staff_only and not request.user.is_staff:
-                continue
+            required_groups = getattr(
+                view_func, "required_security_groups", frozenset()
+            )
+            blocked_reason = None
+            if required_groups:
+                requires_login = True
+                if not user_is_authenticated:
+                    blocked_reason = "login"
+                elif not user_is_superuser and not (
+                    user_group_names & set(required_groups)
+                ):
+                    blocked_reason = "permission"
+            elif requires_login and not user_is_authenticated:
+                blocked_reason = "login"
+
+            if staff_only and not getattr(request.user, "is_staff", False):
+                if blocked_reason != "login":
+                    blocked_reason = "permission"
+
+            landing.nav_is_locked = bool(blocked_reason)
+            landing.nav_lock_reason = blocked_reason
             landings.append(landing)
         if landings:
+            normalized_module_path = module.path.rstrip("/") or "/"
+            if normalized_module_path == "/read":
+                primary_landings = [
+                    landing
+                    for landing in landings
+                    if landing.path.rstrip("/") == normalized_module_path
+                ]
+                if primary_landings:
+                    landings = primary_landings
+                else:
+                    landings = [landings[0]]
             app_name = getattr(module.application, "name", "").lower()
             if app_name == "awg":
-                module.menu = "Calculate"
+                module.menu = "Calculators"
             elif module.path.rstrip("/").lower() == "/man":
                 module.menu = "Manual"
             module.enabled_landings = landings
@@ -73,14 +115,6 @@ def nav_links(request):
                 ):
                     current_module = module
 
-    datasette_lock = Path(settings.BASE_DIR) / "locks" / "datasette.lck"
-    if datasette_lock.exists():
-        datasette_module = SimpleNamespace(
-            menu_label="Data",
-            path="/data/",
-            enabled_landings=[SimpleNamespace(path="/data/", label="Datasette")],
-        )
-        valid_modules.append(datasette_module)
 
     valid_modules.sort(key=lambda m: m.menu_label.lower())
 
@@ -95,10 +129,8 @@ def nav_links(request):
             except Exception:
                 pass
         if not favicon_url:
-            if node and getattr(node.role, "name", "") == "Control":
-                favicon_url = _CONTROL_FAVICON
-            else:
-                favicon_url = _DEFAULT_FAVICON
+            role_name = getattr(getattr(node, "role", None), "name", "")
+            favicon_url = _ROLE_FAVICONS.get(role_name, _DEFAULT_FAVICON) or _DEFAULT_FAVICON
 
     header_refs_qs = (
         Reference.objects.filter(show_in_header=True)
