@@ -7,7 +7,7 @@ DEFAULT_APPLICATION_DESCRIPTIONS: Dict[str, str] = {
     "awg": "Power, Energy and Cost calculations.",
     "core": "Support for Business Processes and monetization.",
     "ocpp": "Compatibility with Standards and Good Practices.",
-    "nodes": "System and Node-level operations,",
+    "nodes": "System and Node-level operations.",
     "pages": "User QA, Continuity Design and Chaos Testing.",
     "teams": "Identity, Entitlements and Access Controls.",
 }
