@@ -1,17 +1,30 @@
+import base64
+import logging
+from datetime import timedelta
+from pathlib import Path
+
 from django.db import models
+from django.db.models import Q
 from core.entity import Entity
+from core.models import Lead, SecurityGroup
 from django.contrib.sites.models import Site
-from nodes.models import NodeRole
+from nodes.models import ContentSample, NodeRole
 from django.apps import apps as django_apps
+from django.utils import timezone
 from django.utils.text import slugify
-from django.utils.translation import gettext, gettext_lazy as _
+from django.utils.translation import gettext, gettext_lazy as _, get_language_info
 from importlib import import_module
 from django.urls import URLPattern
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.validators import MaxLengthValidator, MaxValueValidator, MinValueValidator
+from django.core.exceptions import ValidationError
 
 from core import github_issues
+from .tasks import create_user_story_github_issue
+
+
+logger = logging.getLogger(__name__)
 
 
 class ApplicationManager(models.Manager):
@@ -202,6 +215,7 @@ class Landing(Entity):
     path = models.CharField(max_length=200)
     label = models.CharField(max_length=100)
     enabled = models.BooleanField(default=True)
+    track_leads = models.BooleanField(default=False)
     description = models.TextField(blank=True)
 
     objects = LandingManager()
@@ -223,7 +237,179 @@ class Landing(Entity):
         super().save(*args, **kwargs)
 
 
+class LandingLead(Lead):
+    landing = models.ForeignKey(
+        "pages.Landing", on_delete=models.CASCADE, related_name="leads"
+    )
+
+    class Meta:
+        verbose_name = _("Landing Lead")
+        verbose_name_plural = _("Landing Leads")
+
+    def __str__(self) -> str:  # pragma: no cover - simple representation
+        return f"{self.landing.label} ({self.path})"
+
+
+class RoleLandingManager(models.Manager):
+    def get_by_natural_key(
+        self,
+        role: str | None,
+        group: str | None,
+        username: str | None,
+        module_path: str,
+        path: str,
+    ):
+        filters = {
+            "landing__module__path": module_path,
+            "landing__path": path,
+        }
+        if role:
+            filters["node_role__name"] = role
+        else:
+            filters["node_role__isnull"] = True
+        if group:
+            filters["security_group__name"] = group
+        else:
+            filters["security_group__isnull"] = True
+        if username:
+            filters["user__username"] = username
+        else:
+            filters["user__isnull"] = True
+        return self.get(**filters)
+
+
+class RoleLanding(Entity):
+    node_role = models.OneToOneField(
+        NodeRole,
+        on_delete=models.CASCADE,
+        related_name="default_landing",
+        null=True,
+        blank=True,
+    )
+    security_group = models.OneToOneField(
+        SecurityGroup,
+        on_delete=models.CASCADE,
+        related_name="default_landing",
+        null=True,
+        blank=True,
+    )
+    user = models.OneToOneField(
+        settings.AUTH_USER_MODEL,
+        on_delete=models.CASCADE,
+        related_name="default_landing",
+        null=True,
+        blank=True,
+    )
+    landing = models.ForeignKey(
+        Landing,
+        on_delete=models.CASCADE,
+        related_name="role_defaults",
+    )
+    priority = models.IntegerField(default=0)
+
+    objects = RoleLandingManager()
+
+    class Meta:
+        verbose_name = _("Default Landing")
+        verbose_name_plural = _("Default Landings")
+        ordering = ("-priority", "pk")
+        constraints = [
+            models.CheckConstraint(
+                name="pages_rolelanding_single_target",
+                condition=(
+                    Q(
+                        node_role__isnull=False,
+                        security_group__isnull=True,
+                        user__isnull=True,
+                    )
+                    | Q(
+                        node_role__isnull=True,
+                        security_group__isnull=False,
+                        user__isnull=True,
+                    )
+                    | Q(
+                        node_role__isnull=True,
+                        security_group__isnull=True,
+                        user__isnull=False,
+                    )
+                ),
+            )
+        ]
+
+    def __str__(self) -> str:  # pragma: no cover - simple representation
+        if self.node_role_id:
+            role_name = self.node_role.name
+        elif self.security_group_id:
+            role_name = self.security_group.name
+        elif self.user_id:
+            role_name = self.user.get_username()
+        else:  # pragma: no cover - guarded by constraint
+            role_name = "?"
+        landing_path = self.landing.path if self.landing_id else "?"
+        return f"{role_name} → {landing_path}"
+
+    def natural_key(self):  # pragma: no cover - simple representation
+        role_name = None
+        group_name = None
+        username = None
+        if getattr(self, "node_role_id", None):
+            role_name = self.node_role.name
+        if getattr(self, "security_group_id", None):
+            group_name = self.security_group.name
+        if getattr(self, "user_id", None):
+            username = self.user.get_username()
+        landing_key = (None, None)
+        if getattr(self, "landing_id", None):
+            landing_key = (
+                self.landing.module.path if self.landing.module_id else None,
+                self.landing.path,
+            )
+        return (role_name, group_name, username) + landing_key
+
+    natural_key.dependencies = [
+        "nodes.NodeRole",
+        "core.SecurityGroup",
+        settings.AUTH_USER_MODEL,
+        "pages.Landing",
+    ]
+
+    def clean(self):
+        super().clean()
+        targets = [
+            bool(self.node_role_id),
+            bool(self.security_group_id),
+            bool(self.user_id),
+        ]
+        if sum(targets) == 0:
+            raise ValidationError(
+                {
+                    "node_role": _("Select a node role, security group, or user."),
+                    "security_group": _(
+                        "Select a node role, security group, or user."
+                    ),
+                    "user": _("Select a node role, security group, or user."),
+                }
+            )
+        if sum(targets) > 1:
+            raise ValidationError(
+                {
+                    "node_role": _(
+                        "Only one of node role, security group, or user may be set."
+                    ),
+                    "security_group": _(
+                        "Only one of node role, security group, or user may be set."
+                    ),
+                    "user": _(
+                        "Only one of node role, security group, or user may be set."
+                    ),
+                }
+            )
+
 class UserManual(Entity):
+    class PdfOrientation(models.TextChoices):
+        LANDSCAPE = "landscape", _("Landscape")
+        PORTRAIT = "portrait", _("Portrait")
+
     slug = models.SlugField(unique=True)
     title = models.CharField(max_length=200)
     description = models.CharField(max_length=200)
@@ -235,6 +421,12 @@ class UserManual(Entity):
     )
     content_html = models.TextField()
     content_pdf = models.TextField(help_text="Base64 encoded PDF")
+    pdf_orientation = models.CharField(
+        max_length=10,
+        choices=PdfOrientation.choices,
+        default=PdfOrientation.LANDSCAPE,
+        help_text=_("Orientation used when rendering the PDF download."),
+    )
 
     class Meta:
         db_table = "man_usermanual"
@@ -247,6 +439,39 @@ class UserManual(Entity):
     def natural_key(self):  # pragma: no cover - simple representation
         return (self.slug,)
 
+    def _ensure_pdf_is_base64(self) -> None:
+        """Normalize ``content_pdf`` so stored values are base64 strings."""
+
+        value = self.content_pdf
+        if value in {None, ""}:
+            self.content_pdf = "" if value is None else value
+            return
+
+        if isinstance(value, (bytes, bytearray, memoryview)):
+            self.content_pdf = base64.b64encode(bytes(value)).decode("ascii")
+            return
+
+        reader = getattr(value, "read", None)
+        if callable(reader):
+            data = reader()
+            if hasattr(value, "seek"):
+                try:
+                    value.seek(0)
+                except Exception:  # pragma: no cover - best effort reset
+                    pass
+            self.content_pdf = base64.b64encode(data).decode("ascii")
+            return
+
+        if isinstance(value, str):
+            stripped = value.strip()
+            if stripped.startswith("data:"):
+                _, _, encoded = stripped.partition(",")
+                self.content_pdf = encoded.strip()
+
+    def save(self, *args, **kwargs):
+        self._ensure_pdf_is_base64()
+        super().save(*args, **kwargs)
+
 
 class ViewHistory(Entity):
     """Record of public site visits."""
@@ -267,6 +492,14 @@ class ViewHistory(Entity):
     def __str__(self) -> str:  # pragma: no cover - simple representation
         return f"{self.method} {self.path} ({self.status_code})"
 
+    @classmethod
+    def purge_older_than(cls, *, days: int) -> int:
+        """Delete history entries recorded more than ``days`` days ago."""
+
+        cutoff = timezone.now() - timedelta(days=days)
+        deleted, _ = cls.objects.filter(visited_at__lt=cutoff).delete()
+        return deleted
+
 
 class Favorite(Entity):
     user = models.ForeignKey(
@@ -277,12 +510,14 @@ class Favorite(Entity):
     content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
     custom_label = models.CharField(max_length=100, blank=True)
     user_data = models.BooleanField(default=False)
+    priority = models.IntegerField(default=0)
 
     class Meta:
         unique_together = ("user", "content_type")
+        ordering = ["priority", "pk"]
 
 
-class UserStory(Entity):
+class UserStory(Lead):
     path = models.CharField(max_length=500)
     name = models.CharField(max_length=40, blank=True)
     rating = models.PositiveSmallIntegerField(
@@ -322,6 +557,19 @@ class UserStory(Entity):
         blank=True,
         help_text=_("Link to the GitHub issue created for this feedback."),
     )
+    screenshot = models.ForeignKey(
+        ContentSample,
+        on_delete=models.SET_NULL,
+        blank=True,
+        null=True,
+        related_name="user_stories",
+        help_text=_("Screenshot captured for this feedback."),
+    )
+    language_code = models.CharField(
+        max_length=15,
+        blank=True,
+        help_text=_("Language selected when the feedback was submitted."),
+    )
 
     class Meta:
         ordering = ["-submitted_at"]
@@ -367,6 +615,21 @@ class UserStory(Entity):
             f"**Screenshot requested:** {screenshot_requested}",
         ]
 
+        language_code = (self.language_code or "").strip()
+        if language_code:
+            normalized = language_code.replace("_", "-").lower()
+            try:
+                info = get_language_info(normalized)
+            except KeyError:
+                language_display = ""
+            else:
+                language_display = info.get("name_local") or info.get("name") or ""
+
+            if language_display:
+                lines.append(f"**Language:** {language_display} ({normalized})")
+            else:
+                lines.append(f"**Language:** {normalized}")
+
         if self.submitted_at:
             lines.append(f"**Submitted at:** {self.submitted_at.isoformat()}")
 
@@ -418,6 +681,37 @@ from django.db.models.signals import post_save
 from django.dispatch import receiver
 
 
+def _celery_lock_path() -> Path:
+    return Path(settings.BASE_DIR) / "locks" / "celery.lck"
+
+
+def _is_celery_enabled() -> bool:
+    return _celery_lock_path().exists()
+
+
+@receiver(post_save, sender=UserStory)
+def _queue_low_rating_user_story_issue(
+    sender, instance: UserStory, created: bool, raw: bool, **kwargs
+) -> None:
+    if raw or not created:
+        return
+    if instance.rating >= 5:
+        return
+    if instance.github_issue_url:
+        return
+    if not instance.user_id:
+        return
+    if not _is_celery_enabled():
+        return
+
+    try:
+        create_user_story_github_issue.delay(instance.pk)
+    except Exception:  # pragma: no cover - logging only
+        logger.exception(
+            "Failed to enqueue GitHub issue creation for user story %s", instance.pk
+        )
+
+
 @receiver(post_save, sender=Module)
 def _create_landings(
     sender, instance, created, raw, **kwargs
