@@ -6,7 +6,16 @@ app_name = "pages"
 
 urlpatterns = [
     path("", views.index, name="index"),
+    path(
+        "read/assets/<str:source>/<path:asset>",
+        views.readme_asset,
+        name="readme-asset",
+    ),
+    path("read/<path:doc>/edit/", views.readme_edit, name="readme-edit"),
+    path("read/", views.readme, name="readme"),
+    path("read/<path:doc>", views.readme, name="readme-document"),
     path("sitemap.xml", views.sitemap, name="pages-sitemap"),
+    path("release/", views.release_admin_redirect, name="release-admin"),
     path("client-report/", views.client_report, name="client-report"),
     path("release-checklist", views.release_checklist, name="release-checklist"),
     path("login/", views.login_view, name="login"),
@@ -17,7 +26,6 @@ urlpatterns = [
         views.invitation_login,
         name="invitation-login",
     ),
-    path("datasette-auth/", views.datasette_auth, name="datasette-auth"),
     path("man/", views.manual_list, name="manual-list"),
     path("man/<slug:slug>/", views.manual_detail, name="manual-detail"),
     path("man/<slug:slug>/pdf/", views.manual_pdf, name="manual-pdf"),
