@@ -12,6 +12,8 @@ from django.test import TestCase
 from django.urls import reverse
 from django.contrib.auth import get_user_model
 
+from core.models import ReleaseManager
+
 
 class AdminProfileLinkTests(TestCase):
     def setUp(self):
@@ -28,3 +30,15 @@ class AdminProfileLinkTests(TestCase):
         expected_url = reverse("admin:teams_user_change", args=[self.user.pk])
         self.assertContains(response, "Active Profile")
         self.assertContains(response, f'href="{expected_url}"')
+
+    def test_profile_link_shows_unset_when_missing(self):
+        response = self.client.get(reverse("admin:index"))
+        self.assertContains(response, "Active Profile (Unset)")
+
+    def test_profile_link_shows_profile_name_when_available(self):
+        profile = ReleaseManager.objects.create(user=self.user)
+        response = self.client.get(reverse("admin:index"))
+        expected_url = reverse("admin:teams_releasemanager_change", args=[profile.pk])
+        expected_label = f"Active Profile ({profile})"
+        self.assertContains(response, expected_label)
+        self.assertContains(response, f'href="{expected_url}"')
