@@ -3,7 +3,7 @@ from pathlib import Path
 from django.test import TestCase
 from django.urls import reverse
 
-from core.models import Reference
+from core.models import PackageRelease, Reference
 from core.release import DEFAULT_PACKAGE
 from utils import revision
 
@@ -14,7 +14,15 @@ class FooterNoReferencesTests(TestCase):
         response = self.client.get(reverse("pages:login"))
         self.assertContains(response, "<footer", html=False)
         version = Path("VERSION").read_text().strip()
-        rev_short = revision.get_revision()[-6:]
+        revision_value = (revision.get_revision() or "").strip()
+        release = PackageRelease.objects.filter(version=version).first()
+        release_revision = ""
+        if release and release.revision:
+            release_revision = release.revision.strip()
+        rev_short = ""
+        if revision_value and revision_value != release_revision:
+            rev_short = revision_value[-6:]
+
         release_name = f"{DEFAULT_PACKAGE.name}-{version}"
         if rev_short:
             release_name = f"{release_name}-{rev_short}"
