@@ -5,8 +5,10 @@ import shutil
 import importlib.util
 from glob import glob
 import io
+import uuid
 from contextlib import redirect_stdout
 from pathlib import Path
+from unittest.mock import patch
 
 sys.path.append(str(Path(__file__).resolve().parent.parent))
 os.environ.setdefault("DJANGO_SETTINGS_MODULE", "config.settings")
@@ -22,10 +24,13 @@ from teams.models import OdooProfile, SecurityGroup
 from django.contrib.sites.models import Site
 from django.urls import reverse
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import Group
 from django.core.management import call_command
+from django.utils import timezone
 import socket
 from core.models import Brand, Todo, WMICode
 from core.user_data import dump_user_fixture
+from pages.models import Application, Module, Landing
 
 
 class SeedDataEntityTests(TestCase):
@@ -90,8 +95,14 @@ class EntityInheritanceTests(TestCase):
 
         allowed = {
             "core.SecurityGroup",
+            "core.TOTPDeviceSettings",
+            "ocpp.DataTransferMessage",
+            "ocpp.ChargerConfiguration",
+            "nodes.PendingNetMessage",
+            "pages.Manual",
             "pages.SiteProxy",
             "teams.SecurityGroup",
+            "teams.TOTPDevice",
         }
         for app_label in getattr(settings, "LOCAL_APPS", []):
             config = apps.get_app_config(app_label)
@@ -204,6 +215,60 @@ class EnvRefreshFixtureTests(TestCase):
                         "model": "nodes.noderole",
                         "pk": 999,
                         "fields": {"name": "Fixture Role"},
+                    },
+                    {
+                        "model": "auth.group",
+                        "pk": 777,
+                        "fields": {"name": "Fixture Group"},
+                    },
+                ]
+            )
+        )
+        rel_path = str(fixture_path.relative_to(base_dir))
+        spec = importlib.util.spec_from_file_location(
+            "env_refresh", base_dir / "env-refresh.py"
+        )
+        env_refresh = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(env_refresh)
+        env_refresh._fixture_files = lambda: [rel_path]
+        from django.core.management import call_command as django_call
+
+        loaded_fixtures: list[str] = []
+
+        def fake_call_command(name, *args, **kwargs):
+            if name == "loaddata":
+                loaded_fixtures.extend(args)
+                django_call(name, *args, **kwargs)
+            # ignore other commands
+
+        env_refresh.call_command = fake_call_command
+        buf = io.StringIO()
+        with redirect_stdout(buf):
+            env_refresh.run_database_tasks()
+        output = buf.getvalue()
+        role = NodeRole.all_objects.get(pk=999)
+        self.assertTrue(role.is_seed_data)
+        group = Group.objects.get(pk=777)
+        self.assertFalse(hasattr(group, "is_seed_data"))
+        self.assertIn(".", output)
+        self.assertIn("nodes.NodeRole: 1", output)
+        self.assertEqual(len(loaded_fixtures), 1)
+        self.assertNotEqual(loaded_fixtures[0], str(fixture_path))
+        shutil.rmtree(tmp_dir)
+
+    def test_env_refresh_passthrough_fixture_without_changes(self):
+        base_dir = Path(settings.BASE_DIR)
+        tmp_dir = base_dir / "temp_fixture_passthrough"
+        fixture_dir = tmp_dir / "fixtures"
+        fixture_dir.mkdir(parents=True, exist_ok=True)
+        fixture_path = fixture_dir / "sample_passthrough.json"
+        fixture_path.write_text(
+            json.dumps(
+                [
+                    {
+                        "model": "nodes.noderole",
+                        "pk": 1000,
+                        "fields": {"name": "Passthrough Role", "is_seed_data": True},
                     }
                 ]
             )
@@ -217,8 +282,11 @@ class EnvRefreshFixtureTests(TestCase):
         env_refresh._fixture_files = lambda: [rel_path]
         from django.core.management import call_command as django_call
 
+        loaded_fixtures: list[str] = []
+
         def fake_call_command(name, *args, **kwargs):
             if name == "loaddata":
+                loaded_fixtures.extend(args)
                 django_call(name, *args, **kwargs)
             # ignore other commands
 
@@ -226,13 +294,72 @@ class EnvRefreshFixtureTests(TestCase):
         buf = io.StringIO()
         with redirect_stdout(buf):
             env_refresh.run_database_tasks()
-        output = buf.getvalue()
-        role = NodeRole.all_objects.get(pk=999)
+        role = NodeRole.all_objects.get(pk=1000)
         self.assertTrue(role.is_seed_data)
-        self.assertIn(".", output)
-        self.assertIn("nodes.NodeRole: 1", output)
+        self.assertEqual(len(loaded_fixtures), 1)
+        self.assertEqual(loaded_fixtures[0], str(fixture_path))
         shutil.rmtree(tmp_dir)
 
+    def test_env_refresh_preserves_non_fixture_sites(self):
+        base_dir = Path(settings.BASE_DIR)
+        spec = importlib.util.spec_from_file_location(
+            "env_refresh_sites", base_dir / "env-refresh.py"
+        )
+        env_refresh = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(env_refresh)
+
+        site_fixtures = [
+            str(path.relative_to(base_dir))
+            for path in (base_dir / "core" / "fixtures").glob(
+                "references__00_site_*.json"
+            )
+        ]
+        env_refresh._fixture_files = lambda: site_fixtures
+
+        loaded_fixtures: list[str] = []
+
+        def fake_call_command(name, *args, **kwargs):
+            if name == "loaddata":
+                loaded_fixtures.extend(args)
+            return None
+
+        env_refresh.call_command = fake_call_command
+
+        Site.objects.all().delete()
+        Site.objects.create(domain="127.0.0.1", name="Old Local")
+        Site.objects.create(domain="operator.example", name="Operator Portal")
+
+        env_refresh.run_database_tasks()
+
+        self.assertIn("operator.example", Site.objects.values_list("domain", flat=True))
+        self.assertEqual(
+            Site.objects.get(domain="operator.example").name, "Operator Portal"
+        )
+
+        Site.objects.clear_cache()
+        current_site = Site.objects.get_current()
+        self.assertEqual(current_site.domain, "127.0.0.1")
+        self.assertEqual(
+            Site.objects.filter(domain=current_site.domain).count(),
+            1,
+        )
+
+        expected_sites = {
+            "arthexis.com": "Arthexis",
+            "127.0.0.1": "Local",
+            "10.42.0.1": "Router",
+            "192.168.129.10": "Gateway",
+        }
+        for domain, name in expected_sites.items():
+            site = Site.objects.get(domain=domain)
+            self.assertEqual(site.name, name)
+
+        self.assertEqual(len(loaded_fixtures), 0)
+        self.assertEqual(
+            Site.objects.count(),
+            len(expected_sites) + 1,  # fixture domains plus custom site
+        )
+
 
 class EnvRefreshNodeTests(TestCase):
     def setUp(self):
@@ -277,6 +404,114 @@ class EnvRefreshNodeTests(TestCase):
             control_lock.unlink(missing_ok=True)
 
 
+class EnvRefreshLandingTests(TestCase):
+    def setUp(self):
+        base_dir = Path(settings.BASE_DIR)
+        spec = importlib.util.spec_from_file_location(
+            "env_refresh_landings", base_dir / "env-refresh.py"
+        )
+        self.env_refresh = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(self.env_refresh)
+        self.env_refresh._fixture_files = lambda: []
+        self.env_refresh.call_command = lambda *args, **kwargs: None
+
+    def test_env_refresh_marks_landings_seed_data_once(self):
+        role = NodeRole.objects.create(name=f"LandingRole-{uuid.uuid4().hex}")
+        app = Application.objects.create(
+            name=f"landing_app_{uuid.uuid4().hex}", description=""
+        )
+        module = Module.objects.create(
+            node_role=role,
+            application=app,
+            path="/landing-module/",
+        )
+        landing, _ = Landing.objects.get_or_create(
+            module=module,
+            path="/landing-module/",
+            defaults={"label": "Landing", "description": ""},
+        )
+        landing.is_seed_data = False
+        landing.save(update_fields=["is_seed_data"])
+
+        original_update = self.env_refresh.Landing.objects.update
+        update_calls: list[tuple[tuple, dict]] = []
+
+        def tracking_update(*args, **kwargs):
+            update_calls.append((args, kwargs))
+            return original_update(*args, **kwargs)
+
+        with patch.object(
+            self.env_refresh.Landing.objects,
+            "update",
+            side_effect=tracking_update,
+        ):
+            self.env_refresh.run_database_tasks()
+
+        landing.refresh_from_db()
+        self.assertTrue(landing.is_seed_data)
+        self.assertEqual(len(update_calls), 1)
+
+
+class EnvRefreshTodoFixtureTests(TestCase):
+    def setUp(self):
+        base_dir = Path(settings.BASE_DIR)
+        spec = importlib.util.spec_from_file_location(
+            "env_refresh_todos", base_dir / "env-refresh.py"
+        )
+        self.env_refresh = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(self.env_refresh)
+        self.base_dir = base_dir
+        self.fixture_root = base_dir / f"tmp_env_refresh_{uuid.uuid4().hex}"
+        self.fixture_root.mkdir(exist_ok=True)
+        self.addCleanup(lambda: shutil.rmtree(self.fixture_root, ignore_errors=True))
+        self._commands: list[tuple[str, tuple, dict]] = []
+        self._original_call_command = self.env_refresh.call_command
+
+        def _fake_call_command(name, *args, **kwargs):
+            self._commands.append((name, args, kwargs))
+            return None
+
+        self.env_refresh.call_command = _fake_call_command
+        self.addCleanup(self._restore_call_command)
+        self._original_fixture_files = self.env_refresh._fixture_files
+        self.addCleanup(self._restore_fixture_files)
+
+    def _restore_call_command(self):
+        self.env_refresh.call_command = self._original_call_command
+
+    def _restore_fixture_files(self):
+        self.env_refresh._fixture_files = self._original_fixture_files
+
+    def test_env_refresh_skips_soft_deleted_todos(self):
+        request_text = "Validate screen Example"
+        todo = Todo.objects.create(request=request_text, is_seed_data=True)
+        todo.done_on = timezone.now()
+        todo.save(update_fields=["done_on"])
+        todo.delete()
+        fixture_path = self.fixture_root / "todos__validate_screen_example.json"
+        fixture_data = [
+            {
+                "model": "core.todo",
+                "fields": {
+                    "request": request_text,
+                    "url": "/admin/",
+                    "request_details": "",
+                },
+            }
+        ]
+        fixture_path.write_text(json.dumps(fixture_data, indent=2) + "\n", encoding="utf-8")
+        relative = fixture_path.relative_to(self.base_dir)
+        self.env_refresh._fixture_files = lambda: [str(relative)]
+
+        self.env_refresh.run_database_tasks()
+
+        self.assertFalse(any(name == "loaddata" for name, _, _ in self._commands))
+        self.assertFalse(Todo.objects.filter(request=request_text).exists())
+        self.assertTrue(
+            Todo.all_objects.filter(request=request_text, is_deleted=True).exists()
+        )
+
+
 class EnvRefreshUserDataTests(TransactionTestCase):
     def setUp(self):
         base_dir = Path(settings.BASE_DIR)
@@ -355,5 +590,5 @@ class SeedDataViewTests(TestCase):
         names = set(NodeRole.objects.values_list("name", flat=True))
         self.assertEqual(
             names,
-            {"Terminal", "Constellation", "Control", "Satellite"},
+            {"Terminal", "Watchtower", "Control", "Satellite", "Interface"},
         )
