@@ -14,5 +14,5 @@ def test_core_url_autodiscovered():
 
 
 def test_rfid_url_autodiscovered():
-    match = resolve("/ocpp/rfid/")
+    match = resolve("/ocpp/rfid/validator/")
     assert match.view_name == "rfid-reader"
